package physbeans.editors;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import physbeans.inout.*;
import physbeans.math.*;

/**
 * The panel for the MatrixCustomEditor.
 * <p>It shows a list of the array values and allows to change them and the size
 * of the vector.
 */
public class MatrixCustomEditor extends JPanel {
  
  protected PropertyEditorSupport editor;
  protected NumericTextField[][] textfield;
  protected NumericTextField nColsField, nRowsField;
  protected JPanel pList;
  protected Font standardFont;
  
  protected final int COLS = 8;   // size of text fields
  protected final int PREC = 6;   // precision of text fields
  protected final int ROWPIX = 150; 
  protected final int COLPIX = 30; 
  
  
  
  public MatrixCustomEditor(PropertyEditorSupport ed) {
    // get the old values
    editor = ed;
    Matrix value = (Matrix) editor.getValue();
    int m = (value == null) ? 0 : value.getRowDimension();
    int n = (value == null) ? 0 : value.getColumnDimension();
    
    // vertical list with values, on top box for dimension
    standardFont = new Font("Lucida Sans", Font.PLAIN, 11);
    setLayout(new BorderLayout());
    JPanel pHead = new JPanel();
    nColsField = new NumericTextField(m, 8, 4, "no. columns = ", "");
    nColsField.setFont(standardFont);
    pHead.add(nColsField);
    nColsField.addPropertyChangeListener(new MyDimListener());
    
    nRowsField = new NumericTextField(m, 8, 4, "no. rows = ", "");
    nRowsField.setFont(standardFont);
    pHead.add(nRowsField);
    nRowsField.addPropertyChangeListener(new MyDimListener());
    
    // create panel with text fields
    pList = new JPanel(new GridLayout(m, n, 5, 5));
    pList.setPreferredSize(new Dimension(n*ROWPIX+30, m*COLPIX+30));
    JScrollPane scrollPane = new JScrollPane(pList,
        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
        JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    textfield = new NumericTextField[0][0];
    setTextFieldsToValue(value);
    
    add(pHead, BorderLayout.NORTH);
    add(scrollPane, BorderLayout.CENTER);
    setMaximumSize(new Dimension(400, 300));
  }
  
  /**
   * Called when the value of a textfield is changed.
   */
  protected void changeValue() {
    Matrix newValue = getValueFromTextFields();
    
    editor.setValue(newValue);
    editor.firePropertyChange();
  }
  
  /**
   * Called when a value of the dimension fields is changed.
   */
  protected void resize() {
    int m = (int) Math.round(nRowsField.getValue());
    int n = (int) Math.round(nColsField.getValue());
    
    if ((m < 0) || (n < 0)) {
      m = textfield.length;
      n = textfield[0].length;
    }
    
    if ((m != textfield.length)||(n != textfield[0].length)) {
      Matrix newValue = new Matrix(m, n);
      setTextFieldsToValue(newValue);
      editor.setValue(newValue);
      editor.firePropertyChange();
    }
    
    nRowsField.setValueInternally(m);   // to fix fractional input
    nColsField.setValueInternally(n);
  }
  
  /**
   * Gets Matrix from the text fields.
   */
  protected Matrix getValueFromTextFields() {
    int m = textfield.length;
    int n = textfield[0].length;
    Matrix newValues = new Matrix(m, n);
    
    for (int i = 0; i < m; i++) {
      for (int j = 0; j < n; j++) {
        newValues.set(i, j, textfield[i][j].getValue());
      }
    }
    
    return newValues;
  }
  
  /**
   * Sets text fields from Matrix.
   */
  protected void setTextFieldsToValue(Matrix newVal) {
    int m = (newVal == null) ? 0 : newVal.getRowDimension();
    int n = (newVal == null) ? 0 : newVal.getColumnDimension();
    
    if ((textfield.length == m) && (textfield[0].length == n)) {
      for (int i = 0; i < m; i++) {
        for (int j = 0; j < n; j++) {
          textfield[i][j].setValueInternally(newVal.get(i,j));
        }
      }
    } else {
      pList.removeAll();
      pList.setLayout(new GridLayout(m, n, 5, 5));
      textfield = new NumericTextField[m][n];
      for (int i = 0; i < m; i++) {
        for (int j = 0; j < n; j++) {
          textfield[i][j] = new NumericTextField(newVal.get(i,j), COLS, PREC,
              "A(" + i + "," + j + ") = ", "");
          textfield[i][j].setFont(standardFont);
          pList.add(textfield[i][j]);
          textfield[i][j].addPropertyChangeListener(new MyTextListener());
        }
      }
      pList.setPreferredSize(new Dimension(n*ROWPIX+30, m*COLPIX+30));
    }
    validate();
  }
  
  /**
   * Listeners to change in the text fields.
   */
  public class MyTextListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      changeValue();
    }
  }
  
  /**
   * Listeners to change in the dimension field.
   */
  public class MyDimListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      resize();
    }
  }
  
}
