package physbeans.editors;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.beans.*;
import physbeans.inout.*;
import physbeans.math.*;
import physbeans.model.*;

/**
 * The panel inside the Point2dVectorEditor. It contains
 * a list of the array values as pairs of x and y values.
 */
public class Point2dVectorCustomEditor extends JPanel {
  
  protected PropertyEditorSupport editor;
  protected NumericTextField[] textfieldX, textfieldY;
  protected NumericTextField dimField;
  protected Box pList;
  protected Font standardFont;
  
  protected final int cols = 12;   // size of text fields
  protected final int prec = 8;   // precision of text fields
  
  public Point2dVectorCustomEditor(PropertyEditorSupport ed) {
    // get the old values
    editor = ed;
    Point2dVector value = (Point2dVector) editor.getValue();
    int dim = value.size();
    DVector xValues = value.getXCoords();
    DVector yValues = value.getYCoords();
    
    // vertical list with values, on top box for dimension
    standardFont = new Font("Lucida Sans", Font.PLAIN, 11);
    setLayout(new BorderLayout());
    JPanel pHead = new JPanel();
    dimField = new NumericTextField(dim, 8, 4, "Dimension = ", "");
    dimField.setFont(standardFont);
    pHead.add(dimField);
    dimField.addPropertyChangeListener(new MyDimListener());
    
    // create panel with text fields
    // layout: horizontal boxes with x-y pairs, stacked in a vertical box
    pList = new Box(BoxLayout.Y_AXIS);
    pList.setPreferredSize(new Dimension(360, dim*30+30));
    JScrollPane scrollPane = new JScrollPane(pList,
        JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
        JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
    textfieldX = new NumericTextField[0];
    setTextFieldsToVector(value);
    
    add(pHead, BorderLayout.NORTH);
    add(scrollPane, BorderLayout.CENTER);
    setMaximumSize(new Dimension(380, 150));
  }
  
  /**
   * Called when the value of a textfield is changed.
   */
  protected void changeValue() {
    Point2dVector newValue = getValueFromTextFields();
    
    editor.setValue(newValue);
    editor.firePropertyChange();
  }
  
  /**
   * Called when the value of the dimension field is changed.
   */
  protected void resize() {
    int newDim = (int) Math.round(dimField.getValue());
    
    if (newDim < 0) {
      newDim = textfieldX.length;
    }
    
    if (newDim != textfieldX.length) {
      Point2dVector newValue = new Point2dVector(newDim);
      setTextFieldsToVector(newValue);
      editor.setValue(newValue);
      editor.firePropertyChange();
    }
    
    dimField.setValueInternally(newDim);   // to fix fractional input
  }
  
  /**
   * Gets Point2dVector from the text fields.
   */
  protected Point2dVector getValueFromTextFields() {
    int dim = textfieldX.length;
    DVector newXValues = new DVector(dim);
    DVector newYValues = new DVector(dim);
    
    for (int i = 0; i < dim; i++) {
      newXValues.set(i, textfieldX[i].getValue());
      newYValues.set(i, textfieldY[i].getValue());
    }
    
    return new Point2dVector(newXValues, newYValues);
  }
  
  /**
   * Sets text fields from Point2dVector.
   */
  protected void setTextFieldsToVector(Point2dVector newVal) {
    int dim = newVal.size();
    DVector xCoords = newVal.getXCoords();
    DVector yCoords = newVal.getYCoords();
    
    if (textfieldX.length == dim) {
      for (int i = 0; i < dim; i++) {
        textfieldX[i].setValueInternally(xCoords.get(i));
        textfieldY[i].setValueInternally(yCoords.get(i));
      }
    } else {
      pList.removeAll();
      Box[] xBox = new Box[dim];
      textfieldX = new NumericTextField[dim];
      textfieldY = new NumericTextField[dim];
      
      for (int i = 0; i < dim; i++) {
        xBox[i] = new Box(BoxLayout.X_AXIS);
        textfieldX[i] = new NumericTextField(xCoords.get(i), cols, prec,
            "x(" + i + ") = ", "");
        textfieldY[i] = new NumericTextField(yCoords.get(i), cols, prec,
            "y(" + i + ") = ", "");
        textfieldX[i].setFont(standardFont);
        textfieldY[i].setFont(standardFont);
        
        xBox[i].add(textfieldX[i]);
        xBox[i].add(textfieldY[i]);
        pList.add(xBox[i]);
        textfieldX[i].addPropertyChangeListener(new MyTextListener());
        textfieldY[i].addPropertyChangeListener(new MyTextListener());
      }
      pList.setPreferredSize(new Dimension(500, dim*30+30));
    }
    validate();
  }
  
  
  /**
   * Listeners to change in the text fields.
   */
  public class MyTextListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      changeValue();
    }
  }
  
  /**
   * Listeners to change in the dimension field.
   */
  public class MyDimListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      resize();
    }
  }
}
