package physbeans.func;

import physbeans.event.*;
import physbeans.math.*;

/**
 * Describes a function of n (scalar) input values with m (scalar)
 * output values (vector function).
 * Can be used in three different modi:
 * <ul>
 * <li>Auto triggered: setInputVector sets in, computes and sets out and
 *      informs listeners. </li>
 * <li>Hand triggered: setInputVector sets in, explicit call of trigger computes
 *     and sets out and informs listeners.</li>
 * <li>Stateless: computeFunction provides input values and returns result.
 *     In and out are not set, listeners are not called.</li>
 * </ul>
 */
public abstract class GenericNdFunction extends GenericFunction {
  
  // stores the results (out values) internally as bound indexed property
  // stores the input values internally
  //   advantage: allows assembling of input from scalar input channels
  //   disadvantage: more copying of parameters
  //                 should be harmless, since only vector references are passed
  
  protected DVector inputValues;
  protected DVector outputValues;
  
  /**
   * default constructor: 2 in, 2 out
   */
  public GenericNdFunction() {
    this(2,2);
  }
  
  /**
   * general constructor with n input and m output values
   */
  public GenericNdFunction(int nIn, int nOut) {
    super();
    inputValues = new DVector(nIn);
    outputValues = new DVector(nOut);
  }
  
  /**
   * Returns result of function evaluation for given input parameter.
   * Stateless mode: Doesn't affect internal input or output variables,
   *         doesn't send any events.
   */
  abstract protected DVector computeFunction(DVector in);
  
  /**
   * Computes new values for output vector using current value of input.
   * State mode: Sets and uses internal variables.
   */
  protected void compute() {
    outputValues = computeFunction(inputValues);
  }
  
  /**
   * computes output values and informs listeners.<br>
   * input values are checked if requested by checked flag
   */
  public void trigger() {
    compute();  // compute new output values
    fireVectorEvent(new VectorEvent(this, outputValues)); // inform listeners
  }
  
  /**
   * gets the length of the inputValues array
   */
  public int getInputLength() {
    return inputValues.getDimension();
  }
  
  /**
   * Sets the number of input values.<br>
   * usually used at design time
   */
  public void setInputLength(int size) {
    inputValues = new DVector(size);
  }
  
  /**
   * gets the length of the outputValues array
   */
  public int getOutputLength() {
    return outputValues.getDimension();
  }
  
  /**
   * Sets the number of output values.<br>
   * usually used at design time
   */
  public void setOutputLength(int size) {
    outputValues = new DVector(size);
  }
  
  /**
   * Indexed getter for property inputValues.
   * @param index Index of the property.
   * @return Value of the property at <CODE>index</CODE>.
   */
  public double getInputValues(int index) {
    return this.inputValues.get(index);
  }
  
  /**
   * Getter for property inputValues.
   * @return Value of property inputValues.
   */
  public DVector getInputVector() {
    return inputValues;
  }
  
  /**
   * sets new input value<br>
   * triggers function if autoTriggered
   */
  public void setInputValues(int index, double val) {
    inputValues.set(index, val);
    
    if (autoTriggered) {
      trigger();
    }
  }
  
  /**
   * sets new input value<br>
   * triggers function if autoTriggered<br>
   * Different name to overcome NetBeans problems with indexed functions.
   */
  public void singleInputValue(int index, double inputValue) {
    setInputValues(index, inputValue);
  }
  
 /**
   * Gets one input value.<br>
   * Different name to overcome NetBeans problems with indexed functions.
   */
  public double singleOutputValue(int index) {
    return getOutputValues(index);
  }
  
  /**
   * sets new input value<br>
   * triggers function if autoTriggered
   */
  public void setInputVector(DVector inputVector) {
    inputValues = inputVector;
    
    if (autoTriggered) {
      trigger();
    }
  }
  
  /**
   * Gets the outputValues property as DVector
   * @return The outputValues property value.
   */
  public DVector getOutputVector() {
    return outputValues;
  }
  
  /**
   * Gets the outputValues at given index
   */
  public double getOutputValues(int index) {
    return outputValues.get(index);
  }
  
  /**
   * Gets first output vector component interpreted as scalar.<br>
   * This is used to simplify routing in case of 1d vector output.
   */
  public double getOutputValue() {
    return outputValues.get(0);
  }
  
  /**
   * Sets the size of the output vector to the size of v and presets its values.
   * <P>This can be used to set constant parts of the output vector that are not 
   * overwritten by the compute function.
   */
  public void setOutputVector(DVector v) {
     outputValues = v;
  }
}
