package physbeans.func;

import java.io.*;
import physbeans.math.*;

/**
 * Applies a piecewise linear function to its input vector.
 * The function is defined by two internal vectors, giving x and y coordinates
 * of the corner points. The function outside the corner points is constant
 * (no extrapolation).
 * <P>It has the two additional parameters offset and scale that are used to change
 * the function globally. They are applied after the use of the corner points.
 */
public class PiecewiseLinearFunction extends OneParameterFunction
    implements Serializable {
  
  // x and y have to have the same length.
  // x is increasing
  protected DVector x;    // x coordinates of corner points
  protected DVector y;    // y coordinates of corner points
  
  protected double offset;
  protected double scale;
  
  /**
   * Constructs a standard function
   */
  public PiecewiseLinearFunction() {
    super(-1);
    x = new DVector(0.0, 0.0);
    y = new DVector(0.0, 1.0);
    offset = 0.0;
    scale = 1.0;
  }
  
  /**
   * Gets x coordinates of corner points.
   */
  public DVector getX() {
    return this.x;
  }
  
  /**
   * Sets x coordinates of corner points.
   * <P>The values have to be given in increasing order.
   */
  public void setX(DVector x) {
    this.x = x;
  }
  
  /**
   * Gets y coordinates of corner points.
   */
  public DVector getY() {
    return this.y;
  }
  
  /**
   * Sets y coordinates of corner points.
   */
  public void setY(DVector y) {
    this.y = y;
  }
  
  /**
   * Gets the offset.
   */
  public double getOffset() {
    return this.offset;
  }
  
  /**
   * Sets the offset.
   */
  public void setOffset(double offset) {
    this.offset = offset;
  }
  
  /**
   * Gets the scale.
   */
  public double getScale() {
    return this.scale;
  }
  
  /**
   * Sets the scale.
   */
  public void setScale(double scale) {
    this.scale = scale;
  }
  
 /*
  * Computes the function value for one given value t.
  */
  public double evaluate(double t) {
    // find t among x values of corner points
    int index = x.binarySearch(t);
    if (index >= 0) {
      // t lies on corner point -> get corresponding y coordinate
      return scale*y.get(index) + offset;
    }
    
    // not found -> interpolate
    double res;
    index = -index - 1;  // cf. return value of binarySearch!
    if (index == 0) {
      // t is left to all corner points
      res = scale*y.get(0) + offset;
    } else if (index == x.getDimension()) {
      // t is right to all corner points
      res = scale*y.get(index - 1) + offset;
    } else {
      // t lies between two points
      double x1 = x.get(index - 1);
      double x2 = x.get(index);
      double y1 = y.get(index - 1);
      double y2 = y.get(index);
      res = scale*((y2 - y1)/(x2 - x1) * (t - x1) + y1) + offset;
    }
    return res;
  }
}
