package physbeans.func;

import physbeans.model.*;
import physbeans.math.*;

/**
 * Function that computes a field value of a given scalar field
 *   at a given fixed or moving position.
 * <P>Input:  time <br>
 * Output: value of ScalarField at current detector position.
 */
public class PointDetector extends Generic1dFunction {
  
  protected DVector pos, pos0, v;
  protected Real2DFunction field;
  protected double tLast;
  
  
  /** Creates a new instance of PointDetector */
  public PointDetector() {
    super(1);          // one input value, one output value
    pos0 = new DVector(0.0, 0.0);
    pos = new DVector(0.0, 0.0);
    v = new DVector(0.0, 0.0);
    tLast = 0.0;
  }
  
  /**
   * Gets the initial position of the detector.
   */
  public DVector getInitialPosition() {
    return pos0;
  }
  
  /**
   * Sets the initial position of the detector.<br>
   * Moves the detector immediately to this position.
   */
  public void setInitialPosition(DVector p) {
    pos0 = p;
    pos = pos0.copy();
    inform();
  }
  
  /**
   * Sets the x component of the initial position.<br>
   * Conveniance method to simplify routing.
   */
  public void setInitialPositionX(double x) {
    setInitialPosition(new DVector(x, getInitialPosition().y()));
  }
  
  /**
   * Sets the y component of the initial position.<br>
   * Conveniance method to simplify routing.
   */
  public void setInitialPositionY(double y) {
    setInitialPosition(new DVector(getInitialPosition().x(), y));
  }
  
  /**
   * Gets the current position of the detector.
   */
  public DVector getPosition() {
    return pos;
  }
  
  /**
   * Gets scalar field to be watched.
   */
  public Real2DFunction getField() {
    return field;
  }
  
  /**
   * Sets scalar field to be watched.
   */
  public void setField(Real2DFunction f) {
    field = f;
    inform();
  }
  
  /**
   * Gets the velocity of the detector.
   */
  public DVector getVelocity() {
    return v;
  }
  
  /**
   * Sets the velocity of the detector.
   */
  public void setVelocity(DVector v) {
    this.v = v;
    inform();
  }
  
  /**
   * Sets the x component of the detector velocity.<br>
   * Conveniance method to simplify routing.
   */
  public void setVelocityX(double vx) {
    v.set(0, vx);
  }
  
  /**
   * Sets the y component of the detector velocity.<br>
   * Conveniance method to simplify routing.
   */
  public void setVelocityY(double vy) {
    v.set(1, vy);
  }
  
  /**
   * Computes the value of the scalar field at the current position of
   * the detector.
   */
  public DVector computeFunction(double t) {
    if (field != null) {
      pos = pos.plus(v.times(t - tLast));
      double val = field.getScalarValue(pos.x(), pos.y());
      tLast = t;
      
      return new DVector(val);
    } else {
      return new DVector(Double.NaN);
    }
  }
  
  /**
   * Sets current detector position to initial value.
   */
  public void restart() {
    pos = pos0.copy();
    tLast = 0.0;
    this.setInputValue(0.0);
  }
  
  /**
   * Sets time.
   * Conveniance function to simplify routing.
   */
  public void setTime(double t) {
    setInputValue(t);
  }
}
