package physbeans.func;

import physbeans.math.*;
import physbeans.model.*;
import java.io.*;

/**
 * Periodic function describing a rectangular pulse between -A and A.
 * External parameter is the pulse width.
 */
public class SquareFunction extends PeriodicPiecewiseLinearFunction
                implements Serializable, PeriodicFunctionWithParameter {
  
  /** Holds value of property pulseWidth. */
  protected double pulseWidth;
  
  /** Holds value of property amplitude. */
  protected double amplitude;
  
  /** Creates a SquareFunction with pulseWidth = 0.5 and amplitude = 1.0 */
  public SquareFunction() {
    super();
    setPulseWidth(0.5);
    setAmplitude(1.0);
  }
  
  /**
   * Gets the pulse width.
   * It is given as relative value, between 0 and 1.
   */
  public double getPulseWidth() {
    return pulseWidth;
  }
  
  /**
   * Sets the pulse width.
   * Input value is clamped to lie between 0 and 1.
   */
  public void setPulseWidth(double p) {
    if (p != 1.0) {
      p = p - Math.floor(p);
    }
    pulseWidth = p;
    setCornerPoints();
  }
  
  /**
   * Sets the period T.
   */
  public void setPeriod(double period) {
    super.setPeriod(period);
    setCornerPoints();
  }
  
  /*
   * Computes corner points for pulse width and amplitude.
   * The points are defined for a fundamental period of 1.
   */
  protected void setCornerPoints() {
    double p = pulseWidth/2.0;
    x = new DVector(p, p, 1.0 - p, 1.0 - p);
    y = new DVector(amplitude, -amplitude, -amplitude, amplitude);
  }

  /**
   * Gets the parameter value.
   */
  public double getParameter() {
    return getPulseWidth();
  }
  
  /**
   * Sets the parameter
   */
  public void setParameter(double p) {
    setPulseWidth(p);
  }
  
  /** 
   * Gets the amplitude of the square function.
   */
  public double getAmplitude() {
    return this.amplitude;
  }
  
  /** 
   * Sets the amplitude of the square function.
   */
  public void setAmplitude(double amplitude) {
    this.amplitude = amplitude;
    setCornerPoints();
  }
  
}
