package physbeans.inout;

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.io.*;
import physbeans.event.*;
import physbeans.inout.PhysicsPanel;


/**
 * Bean packaging a number of radio buttons. <br>
 * Contains some radio buttons with labels and a common title string.
 * The buttons can be arranged in one or two columns.<br>
 * Sends a SwitchEvent with the number of the clicked button.
 */
public class RadioButtonBox extends PhysicsPanel implements Serializable {
  
  /** checkboxes are arranged in two columns */
  protected boolean withTwoColumns;
  
  // components
  protected Vector<JRadioButton> boxes;
  protected JLabel titleLabel;
  protected JPanel boxPanel;
  protected JRadioButton[] cb;
  protected MyButtonListener listener;
  
  /**
   * Creates a new instance of SwitchBox.
   */
  public RadioButtonBox() {
    super();
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    withTwoColumns = false;
    
    // add heading
    titleLabel = new JLabel("Title", JLabel.LEFT);
    titleLabel.setAlignmentX(LEFT_ALIGNMENT);
    add(titleLabel);
    
    // add radio buttons
    boxPanel = new JPanel();
    boxPanel.setAlignmentX(LEFT_ALIGNMENT);
    boxPanel.setOpaque(false);
    
    listener = new MyButtonListener();
    createBoxes(3);
    add(boxPanel);
    
    setAlignmentX(CENTER_ALIGNMENT);
    setBordered(true);
  }
  
  /**
   * Creates count new standard checkboxes.
   */
  protected void createBoxes(int count) {
    boxes = new Vector<JRadioButton>();
    
    if (withTwoColumns) {
      int rows = (count % 2 == 0)? count/2 : (count + 1)/2;
      boxPanel.setLayout(new GridLayout(rows, 2));
    } else {    // all checkboxes in one column
      boxPanel.setLayout(new GridLayout(count, 1));
    }
    
    ButtonGroup theGroup = new ButtonGroup();
    cb = new JRadioButton[count];
    for (int i = 0; i < count; i++) {
      cb[i] = new JRadioButton("" + i + i + i + i); // default labels
      cb[i].setOpaque(false);
      cb[i].addItemListener(listener);
      theGroup.add(cb[i]);
      
      boxes.addElement(cb[i]);
      
      boxPanel.add(cb[i]);
    }
    
    // first box starts checked by default
    setSelected(0);
  }
  
  /**
   * Gets the title string.
   */
  public String getTitle() {
    return titleLabel.getText();
  }
  
  /**
   * Sets the title string.
   */
  public void setTitle(String title) {
    titleLabel.setText(title);
  }
  
  /**
   * Gets the number of checkboxes.
   */
  public int getCount() {
    return boxes.size();
  }
  
  /**
   * Gets number of selected button.
   */
  public int getSelected() {
    for (int i = 0; i < boxes.size(); i++) {
      if (((JRadioButton) boxes.elementAt(i)).isSelected()) {
        return i;
      }
    }
    
    // should not get here!
    // throw(new IllegalStateException("no radio button selected!"));
    return 0;
  }
  
  /**
   * Selects the radio button no. i without firing an event.
   */
  public void setSelectedInternally(int i) {
    ((JRadioButton) boxes.elementAt(i)).setSelected(true);
  }
  
  /**
   * Sets checkbox i and fires an SwitchEvent.
   */
  public void setSelected(int i) {
    setSelectedInternally(i);
    fireSwitchEvent(new SwitchEvent(this,i, true));
  }
  
  /**
   * Sets foreground color.
   */
  public void setForeground(Color c) {
    super.setForeground(c);
    if (boxPanel != null) {
      boxPanel.setForeground(c);
      for (int i = 0; i < boxes.size(); i++) {
        ((JRadioButton) boxes.elementAt(i)).setForeground(c);
      }
    }
  }
  
  /**
   * Gets text label of checkbox i.
   */
  public String getBoxLabel(int i) {
    return ((JRadioButton) boxes.elementAt(i)).getText();
  }
  
  /**
   * Sets text label of checkbox i to text.
   */
  public void setBoxLabel(int i, String text) {
    ((JRadioButton) boxes.elementAt(i)).setText(text);
  }
  
  /**
   * Gets all checkbox labels as String array.
   */
  public String[] getBoxLabel() {
    int count = getCount();
    String[] labels = new String[count];
    for (int i = 0; i < count; i++) {
      labels[i] =  getBoxLabel(i);
    }
    return labels;
  }
  
  /**
   * Sets all checkbox labels with String array.
   */
  public void setBoxLabel(String[] label) {
    int count = label.length;
    
    // create new boxes if necessary
    if (getCount() != count) {
      boxPanel.removeAll();
      createBoxes(count);
    }
    
    for (int i = 0; i < count; i++) {
      setBoxLabel(i, label[i]);
    }
  }
  
  /**
   * Gets number of selected button as double value.<br>
   * Conveniance method to simplify routing.
   */
  public double getSelectedAsDouble() {
    return (double)getSelected();
  }
  
  /** one listener for all checkboxes in the SwitchBox */
  protected class MyButtonListener implements ItemListener {
    public void itemStateChanged(ItemEvent e) {
      fireSwitchEvent(new SwitchEvent(this, getSelected(), true));
    }
  }
  
  
  // support SwitchEvents
  protected transient SwitchSupport switchSupport = new SwitchSupport();
  
  /** registers a listener to SwitchEvents */
  public synchronized void addSwitchListener(SwitchListener l) {
    switchSupport.addSwitchListener(l);
  }
  
  /** de-registers a listener to SwitchEvents */
  public synchronized void removeSwitchListener(SwitchListener l) {
    switchSupport.removeSwitchListener(l);
  }
  
  /** fires a SwitchEvent */
  public void fireSwitchEvent(SwitchEvent e) {
    switchSupport.fireSwitchEvent(e);
  }
  
  /**
   * Gets flag whether checkboxes are arranged in two columns.
   */
  public boolean isWithTwoColumns() {
    return this.withTwoColumns;
  }
  
  /**
   * Sets flag whether checkboxes are arranged in two columns.
   */
  public void setWithTwoColumns(boolean withTwoColumns) {
    this.withTwoColumns = withTwoColumns;
    
    // Boxes are not rearranged but simply recreated.
    // Therefore box states have to be transferred to the new boxes.
    String[] oldLabels = getBoxLabel();
    int oldState = getSelected();
    boxPanel.removeAll();
    createBoxes(getCount());
    setBoxLabel(oldLabels);
    setSelected(oldState);
  }
  
  public void doLayout() {
    // AFTER definition of text or setting of value
    // best at layout time so that a font is already defined from the parent
    super.doLayout();
    boxPanel.setFont(null);
    titleLabel.setFont(null);
    
    for (int i = 0; i < cb.length; i++) {
      cb[i].setFont(null);
    }
  }
}
