package physbeans.inout;

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import java.io.*;
import physbeans.event.*;

/**
 * Bean packaging a number of checkboxes. <br>
 * Contains some checkboxes with labels and a common title string. The
 * checkboxes can be arranged in one or two columns.<br>
 * Sends a SwitchEvent with the number of the clicked checkbox
 */
public class SwitchBox extends PhysicsPanel implements Serializable {

  protected boolean withTwoColumns;

  protected Vector<JCheckBox> boxes;
  protected JLabel titleLabel;
  protected JPanel boxPanel;
  protected JCheckBox[] cb;

  protected int lastIndex;
  protected MyCheckListener listener;

  // color of the boxes
  // if there are more lines than colors, this is used cyclically
  protected boolean useBoxColors;
  protected Color[] boxColor = {
    Color.green, Color.magenta, Color.cyan, Color.pink, Color.orange,
    Color.red, Color.blue, Color.yellow
  };

  /**
   * Creates a default SwitchBox with 3 boxes.
   */
  public SwitchBox() {
    super();
    setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
    withTwoColumns = false;
    lastIndex = 0;
    useBoxColors = false;

    // add heading
    titleLabel = new JLabel("Title", JLabel.LEFT);
    titleLabel.setAlignmentX(LEFT_ALIGNMENT);
    add(titleLabel);

    // add checkboxes
    boxPanel = new JPanel();
    boxPanel.setAlignmentX(LEFT_ALIGNMENT);
    listener = new MyCheckListener();
    createBoxes(3);
    add(boxPanel);

    setAlignmentX(CENTER_ALIGNMENT);
    setBordered(true);
  }

  /**
   * Creates count new standard checkboxes.
   */
  protected void createBoxes(int count) {
    boxes = new Vector<JCheckBox>();

    if (withTwoColumns) {
      int rows = (count % 2 == 0) ? count / 2 : (count + 1) / 2;
      boxPanel.setLayout(new GridLayout(rows, 2));
    } else {    // all checkboxes in one column
      boxPanel.setLayout(new GridLayout(count, 1));
    }

    cb = new JCheckBox[count];
    for (int i = 0; i < count; i++) {
      cb[i] = new JCheckBox("(" + i + ")"); // default labels

      cb[i].setOpaque(false);
      cb[i].addItemListener(listener);

      boxes.addElement(cb[i]);

      boxPanel.setOpaque(false);
      boxPanel.add(cb[i]);
    }

    // first box starts checked by default
    ((JCheckBox) boxes.elementAt(0)).setSelected(true);
    colorBoxes();
  }

  /**
   * Gets the title string.
   */
  public String getTitle() {
    return titleLabel.getText();
  }

  /**
   * Sets the title string.
   */
  public void setTitle(String title) {
    titleLabel.setText(title);
  }

  /**
   * Gets the number of checkboxes.
   */
  public int getCount() {
    return boxes.size();
  }

  /**
   * Sets the number of checkboxes.
   */
  public void setCount(int dim) {
    boolean[] newVals = new boolean[dim];
    setSelected(newVals);
  }

  /**
   * Gets selection state of the checkbox no. i.
   */
  public boolean getSelected(int i) {
    return ((JCheckBox) boxes.elementAt(i)).isSelected();
  }

  /**
   * Sets selection state of the checkbox no. i. <br>
   * Doesnt fire an event.
   */
  public void setSelected(int i, boolean selected) {
    ((JCheckBox) boxes.elementAt(i)).setSelected(selected);
  }

  /**
   * Gets all checkbox states as boolean array.
   */
  public boolean[] getSelected() {
    int count = getCount();
    boolean[] states = new boolean[count];
    for (int i = 0; i < count; i++) {
      states[i] = getSelected(i);
    }
    return states;
  }

  /**
   * Sets all checkbox states with boolean array.
   */
  public void setSelected(boolean[] states) {
    int count = states.length;

    // create new boxes if necessary
    if (getCount() != count) {
      boxPanel.removeAll();
      createBoxes(count);
    }

    for (int i = 0; i < count; i++) {
      setSelected(i, states[i]);
    }
  }

  /**
   * Gets selection state of the first checkbox.
   * <P>
   * Conveniance function for switch boxes with only one box.
   */
  public boolean getFirstSelected() {
    return getSelected(0);
  }

  /**
   * Gets selection state of the first checkbox as int value (0 = false).
   * <P>
   * Conveniance function for switch boxes with only one box.
   */
  public int getFirstSelectedAsInt() {
    if (getSelected(0)) {
      return 1;
    } else {
      return 0;
    }
  }

  /**
   * Gets index of last changed switch.
   */
  public int getLastIndex() {
    return lastIndex;
  }

  /**
   * Sets checkbox i and fires an SwitchEvent
   */
  public void doClick(int i, boolean selected) {
    setSelected(i, selected);
    fireSwitchEvent(new SwitchEvent(this, i, false));
  }

  /**
   * Sets foreground color.
   */
  public void setForeground(Color c) {
    super.setForeground(c);
    if (boxPanel != null) {
      boxPanel.setForeground(c);
      for (int i = 0; i < boxes.size(); i++) {
        ((JCheckBox) boxes.elementAt(i)).setForeground(c);
      }
    }
  }

  /**
   * Gets text label of checkbox i.
   */
  public String getBoxLabel(int i) {
    return ((JCheckBox) boxes.elementAt(i)).getText();
  }

  /**
   * Sets text label of checkbox i to text.
   */
  public void setBoxLabel(int i, String text) {
    ((JCheckBox) boxes.elementAt(i)).setText(text);
  }

  /**
   * Gets all checkbox labels as String array.
   */
  public String[] getBoxLabel() {
    int count = getCount();
    String[] labels = new String[count];
    for (int i = 0; i < count; i++) {
      labels[i] = getBoxLabel(i);
    }
    return labels;
  }

  /**
   * Sets all checkbox labels with String array.
   */
  public void setBoxLabel(String[] label) {
    int count = label.length;

    // create new boxes if necessary
    if (getCount() != count) {
      boxPanel.removeAll();
      createBoxes(count);
    }

    for (int i = 0; i < count; i++) {
      setBoxLabel(i, label[i]);
    }
  }

  /**
   * Gets flag whether checkboxes are colored.
   */
  public boolean isUseBoxColors() {
    return this.useBoxColors;
  }

  /**
   * Sets flag whether checkboxes are colored.
   */
  public void setUseBoxColors(boolean useBoxColors) {
    this.useBoxColors = useBoxColors;
    colorBoxes();
  }

  /**
   * Gets color of a box.
   */
  public Color getBoxColor(int index) {
    return this.boxColor[index];
  }

  /**
   * Gets color of all boxes.
   */
  public Color[] getBoxColor() {
    return this.boxColor;
  }

  /**
   * Sets color of a box.
   */
  public void setBoxColor(int index, Color boxColor) {
    this.boxColor[index] = boxColor;
    colorBoxes();
  }

  /**
   * Sets color of all boxes.
   */
  public void setBoxColor(Color[] boxColor) {
    this.boxColor = boxColor;
    colorBoxes();
  }

  /**
   * One listener for all checkboxes in the SwitchBox.
   */
  protected class MyCheckListener implements ItemListener {

    public void itemStateChanged(ItemEvent e) {
      // find the activated checkbox and fire a SwitchEvent
      for (int i = 0; i < boxes.size(); i++) {
        if (e.getSource().equals(boxes.elementAt(i))) {
          boolean state = ((JCheckBox) e.getSource()).isSelected();
          lastIndex = i;
          fireSwitchEvent(new SwitchEvent(this, i, state));
        }
      }
    }
  }

  // support SwitchEvents
  protected transient SwitchSupport switchSupport = new SwitchSupport();

  /**
   * registers a listener to SwitchEvents
   */
  public synchronized void addSwitchListener(SwitchListener l) {
    switchSupport.addSwitchListener(l);
  }

  /**
   * de-registers a listener to SwitchEvents
   */
  public synchronized void removeSwitchListener(SwitchListener l) {
    switchSupport.removeSwitchListener(l);
  }

  /**
   * fires a SwitchEvent
   */
  public void fireSwitchEvent(SwitchEvent e) {
    switchSupport.fireSwitchEvent(e);
  }

  /**
   * Gets flag whether checkboxes are arranged in two columns.
   */
  public boolean isWithTwoColumns() {
    return this.withTwoColumns;
  }

  /**
   * Sets flag whether checkboxes are arranged in two columns.
   */
  public void setWithTwoColumns(boolean withTwoColumns) {
    this.withTwoColumns = withTwoColumns;

    // Boxes are not rearranged but simply recreated.
    // Therefore box states have to be transferred to the new boxes.
    String[] oldLabels = getBoxLabel();
    boolean[] oldStates = getSelected();
    boxPanel.removeAll();
    createBoxes(getCount());
    setBoxLabel(oldLabels);
    setSelected(oldStates);  // fires new (= old) active states, should be ok.
  }

  public void doLayout() {
    // AFTER definition of text or setting of value
    // best at layout time so that a font is already defined from the parent
    super.doLayout();
    boxPanel.setFont(null);
    titleLabel.setFont(null);

    for (int i = 0; i < cb.length; i++) {
      cb[i].setFont(null);
    }
  }

  /**
   * Color boxes according to boxColors.
   */
  protected void colorBoxes() {
    int nBoxes = boxes.size();
    int nColors = boxColor.length;
    if (useBoxColors) {
      for (int i = 0; i < nBoxes; i++) {
        JCheckBox box = (JCheckBox) boxes.elementAt(i);
        box.setBackground(boxColor[i % nColors]);
        box.setOpaque(true);
      }
    }
  }
}
