package physbeans.inout;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import java.beans.*;
import java.io.*;
import java.net.*;
import physbeans.math.*;

/**
 * Bean for input and output of a text line.<br>
 * The text field is optionally surrounded by pre and post texts.
 */
public class TextLine extends PhysicsPanel implements Serializable {
  
  protected String descriptionIcon, postscriptIcon;
  protected JTextField tf;
  protected JLabel descriptionLabel, postscriptLabel;
  
  /**
   * constructs a default TextLine
   */
  public TextLine() {
    this("", 20, "Description", "Postscript");
  }
  
  public TextLine(String val, int cols, String desc, String post) {
    super();
    setLayout(new BoxLayout(this, BoxLayout.X_AXIS ));
    
    // setup description label
    descriptionLabel = new JLabel(desc, JLabel.RIGHT);
    descriptionLabel.setFont(null);
    descriptionLabel.setBorder(BorderFactory.createEmptyBorder(0,0,0,4));
    
    // setup text field
    tf = new JTextField(val, cols);
    tf.setBackground(Color.white);
    tf.setForeground(Color.black);
    tf.addActionListener(new MyTextFieldListener());
    
    // setup postscripts label
    postscriptLabel = new JLabel(post, JLabel.LEFT);
    postscriptLabel.setBorder(BorderFactory.createEmptyBorder(0,4,0,0));
    postscriptLabel.setFont(null);
    
    // build up GUI
    add(descriptionLabel);
    add(tf);
    add(postscriptLabel);
  }
  
  public void doLayout() {
    super.doLayout();
    tf.setFont(null); // AFTER definition of text or setting of value
    // best at layout time so that a font is already defined from the parent
    
  }
  
  /**
   * fire property change event.
   */
  protected void firePropertyChange(String name, Object oldVal, Object newVal) {
    super.firePropertyChange(name, oldVal, newVal);
  }
  
  /**
   * Gets number of columns of text field.
   */
  public int getColumns() {
    return tf.getColumns();
  }
  
  /**
   * Gets description text.
   */
  public String getDescription() {
    return descriptionLabel.getText();
  }
  
  /**
   * Gets postscript text.
   */
  public String getPostscript() {
    return postscriptLabel.getText();
  }
  
  /**
   * Gets current value of text field.
   */
  public String getValue() {
    return tf.getText();
  }
  
  /**
   * True if text field is editable.
   */
  public boolean isEditable() {
    return tf.isEditable();
  }
  
  /**
   * Gets number of columns of text field.
   */
  public void setColumns(int col) {
    tf.setColumns(col);
  }
  
  /**
   * Sets description text.
   */
  public void setDescription(String s) {
    descriptionLabel.setText(s);
  }
  
  /**
   * Gets filename of icon used for the description field.
   */
  public String getDescriptionIcon() {
    return descriptionIcon;
  }
  
  /**
   * Sets filename of icon used for the description field.
   */
  public void setDescriptionIcon(String file) {
    descriptionIcon = file;
    descriptionLabel.setIcon(getIcon(file));
    descriptionLabel.setVerticalTextPosition(SwingConstants.BOTTOM);
  }
  
  /**
   * Gets filename of icon used for the postscript field.
   */
  public String getPostscriptIcon() {
    return postscriptIcon;
  }
  
  /**
   * Sets filename of icon used for the postscript field.
   */
  public void setPostscriptIcon(String file) {
    postscriptIcon = file;
    postscriptLabel.setIcon(getIcon(file));
    postscriptLabel.setVerticalTextPosition(SwingConstants.BOTTOM);
  }
  
  
  /**
   * Sets editable status of text field. <br>
   * true for input field, false for output field.
   */
  public void setEditable(boolean flag) {
    tf.setEditable(flag);
  }
  
  /**
   * Sets postscript text.
   */
  public void setPostscript(String s) {
    postscriptLabel.setText(s);
  }
  
  /**
   * Sets TextField to value.
   */
  public void setValue(String s) {
    tf.setText(s);
    firePropertyChange("value", null, s);
  }
  
  /**
   * Finds and loads icons.
   */
  protected ImageIcon getIcon(String file) {
    ClassLoader myCL = getClass().getClassLoader();
    URL myURL = myCL.getResource(file);
    if (myURL != null) {
      return new ImageIcon(myURL);
    } else {
      return null;
    }
  }
  
  /**
   * listener to text field action. <br>
   * informs PropertyChangeListeners
   */
  protected class MyTextFieldListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      String newValue = getValue();
      firePropertyChange("value", null, newValue);
    }
  }
}
