package physbeans.inout;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import java.beans.*;
import java.io.*;
import physbeans.model.*;
import physbeans.math.*;

/**
 * Bean for input of a vector of numerical values via a set of sliders and
 * a text field.<br>
 * The sliders are arranged horizontally or vertically, they have linear or
 * logarithmic characteristics and can send events either continuously (during
 * drag) or only after drag stopped. All sliders have identical minimal and
 * maximal values, their initial values can be set differently.<br>
 * After a click on a slider or a drag operation the slider is "active", its
 * value is displayed in the text field and can be changed there directly.
 */
public class VectorSlider extends PhysicsPanel implements Serializable {
  
  protected DVector values;
  protected DVector initialValues;
  protected double max, min;
  protected int activeIndex;
  protected boolean vertical, logarithmic, fireContinuously, inverted;
  
  protected NumericTextField textField;
  protected ArrayList<Slider> sliders;
  protected JPanel sliderPanel;
  
  /** Creates a new instance of TextSlider */
  public VectorSlider() {
    super();
    setLayout(new BorderLayout());
    
    min = 0.0;
    max = 1.0;
    vertical = true;
    logarithmic = false;
    fireContinuously = false;
    inverted = false;
    activeIndex = 0;
    initialValues = new DVector(0.1, 0.2, 0.3, 0.4);
    
    textField = new NumericTextField();
    textField.setPrecision(4);   // adapted to slider precision
    textField.setColumns(5);
    textField.addPropertyChangeListener(new MyTextFieldListener());
    add(textField, BorderLayout.NORTH);
    
    sliders = new ArrayList<Slider>();
    sliderPanel = new JPanel();
    sliderPanel.setOpaque(false);
    createSliders(4);
    add(sliderPanel, BorderLayout.CENTER);
    
    setBordered(true);
  }
  
  /**
   * Creates count new standard sliders.
   */
  protected void createSliders(int n) {
    sliderPanel.removeAll();
    sliders.clear();
    values = new DVector(n);
    
    int orientation = vertical ? BoxLayout.X_AXIS : BoxLayout.Y_AXIS;
    sliderPanel.setLayout(new BoxLayout(sliderPanel, orientation));
    
    for (int i = 0; i < n; i++) {
      Slider slider = new Slider();
      slider.setModel(new BoundedRealModel(min, min, max));
      slider.setPaintTicks(false);
      slider.setVertical(vertical);
      slider.setLogarithmic(logarithmic);
      slider.setFireContinuously(fireContinuously);
      slider.setInverted(inverted);
      slider.addPropertyChangeListener(new MySliderListener());
      slider.addMouseListener(new MyMouseListener());
      slider.addFocusListener(new MyFocusListener());
      sliders.add(slider);
      sliderPanel.add(slider);
    }
    sliders.get(activeIndex).setMarked(true);
    sliderPanel.validate();
    setInitialValues(initialValues);
    
  }
  
  /**
   * Gets the current values.
   */
  public DVector getValue() {
    return values;
  }
  
  /**
   * Sets the current values and adjusts the sliders accordingly.<br>
   * Fires a property change event.
   */
  public void setValue(DVector val) {
    setValueInternally(val);
    firePropertyChange("value", null, val);
  }
  
  /**
   * Gets number of sliders.
   */
  public int getCount() {
    return values.getDimension();
  }
  
  /**
   * Sets the numer of sliders.
   */
  public void setCount(int n) {
    if ((getCount() != n) && (n > 0)) {
      initialValues = new DVector(n);
      createSliders(n);
    }
  }
  
  /**
   * Gets index of currently active slider.
   */
  public int getActiveIndex() {
    return activeIndex;
  }
  
  /**
   * Sets index of currently active slider.<br>
   * Usually activeIndex is set by user gestures. It can be set explicitely
   * for initialisation or testing.
   */
  public void setActiveIndex(int n) {
    if (n < 0 || n >= getCount()) {
      return;
    }
    sliders.get(activeIndex).setMarked(false);
    activeIndex = n;
    sliders.get(activeIndex).setMarked(true);
  }
  
  /**
   * Gets value of vertical flag.<br>
   * If true, the sliders are oriented vertically.
   */
  public boolean isVertical() {
    return vertical;
  }
  
  /**
   * Sets value of vertical flag.<br>
   * If true, the sliders are oriented vertically.
   */
  public void setVertical(boolean b) {
    vertical = b;
    createSliders(getCount());
  }
  
  /**
   * Sets the current values and adjusts the sliders accordingly
   * without firing an event.
   * <P>Adjusts the values according to the slider bounds.
   */
  protected void setValueInternally(DVector newVals) {
    int n = getCount();
    if (newVals.getDimension() != n) {
      return;                             // wrong vector length
    }
    
    for (int i = 0; i < n; i++) {
      double val = newVals.get(i);
      val = Math.max(Math.min(val, max), min);
      values.set(i, val);
      sliders.get(i).setValueInternally(val);
    }
    textField.setValueInternally(values.get(activeIndex));
  }
  
  /**
   * Gets description string.
   */
  public String getDescription() {
    return textField.getDescription();
  }
  
  /**
   * Sets description string.
   */
  public void setDescription(String description) {
    textField.setDescription(description);
  }
  
  /**
   * Gets filename of icon used for the description field.
   */
  public String getDescriptionIcon() {
    return textField.getDescriptionIcon();
  }

  /**
   * Sets filename of icon used for the description field.
   */
  public void setDescriptionIcon(String file) {
    textField.setDescriptionIcon(file);
  }

  /**
   * Gets number of digits displayed in text field.
   */
  public int getPrecision() {
    return textField.getPrecision();
  }
  
  /**
   * Sets number of digits displayed in text field.
   */
  public void setPrecision(int precision) {
    textField.setPrecision(precision);
  }
  
  /**
   * Gets number of columns in text field.
   */
  public int getColumns() {
    return textField.getColumns();
  }
  
  /**
   * Sets number of columns in text field.
   */
  public void setColumns(int columns) {
    textField.setColumns(columns);
  }
  
  /**
   * Gets unit string.
   */
  public String getUnit() {
    return textField.getUnit();
  }
  
  /**
   * Sets unit string.
   */
  public void setUnit(String unit) {
    textField.setUnit(unit);
  }
  
  /**
   * True if events are sent continuously during slider drag.
   */
  public boolean isFireContinuously() {
    return fireContinuously;
  }
  
  /**
   * Sets fireContinuously flag.
   */
  public void setFireContinuously(boolean fireContinuously) {
    this.fireContinuously = fireContinuously;
    createSliders(getCount());
  }
  
  /**
   * True if slider scale is logarithmic.
   */
  public boolean isLogarithmic() {
    return logarithmic;
  }
  
  /**
   * Sets logarithmic flag.
   */
  public void setLogarithmic(boolean logarithmic) {
    this.logarithmic = logarithmic;
    createSliders(getCount());
  }
  
  /**
   * Gets minimal value of the sliders.
   */
  public double getMinimum() {
    return sliders.get(0).getModel().getMinimum();
  }
  
  /**
   * Sets minimal value of the sliders.
   */
  public void setMinimum(double min) {
    this.min = min;
    for (Slider slider: sliders) {
      slider.getModel().setMinimum(min);
    }
    setInitialValues(initialValues);
  }
  
  /**
   * Gets maximal value of the sliders.
   */
  public double getMaximum() {
    return sliders.get(0).getModel().getMaximum();
  }
  
  /**
   * Sets maximal value of the sliders.
   */
  public void setMaximum(double max) {
    this.max = max;
    for (Slider slider: sliders) {
      slider.getModel().setMaximum(max);
    }
    setInitialValues(initialValues);
  }
  
  /**
   * Gets initial values of the sliders.
   */
  public DVector getInitialValues() {
    return initialValues;
  }
  
  /**
   * Sets initial values of the sliders.
   * <p>Sets current slider values and informs listeners.
   */
  public void setInitialValues(DVector init) {
    if (init.getDimension() == getCount()) {
      initialValues = init;
      setValue(init);
    }
  }
  
  /**
   * Sets foreground color.
   */
  public void setForeground(Color c) {
    super.setForeground(c);
    if (textField != null) {
      textField.setForeground(c);
    }
    if (sliders != null) {
      for (Slider slider: sliders) {
        slider.setForeground(c);
      }
    }
  }
  
  /**
   * Gets values of inverted flag.<br>
   *  If true, maximal slider values is to the left.
   */
  public boolean getInverted() {
    return inverted;
  }
  
  /**
   * Sets values of inverted flag.<br>
   *  True means maximal slider values is to the left.
   */
  public void setInverted(boolean inverted) {
    this.inverted = inverted;
    createSliders(getCount());
  }
  
  /**
   * Listener to Slider events,<br>
   *  sets text field and fires PropertyChange event
   */
  protected class MySliderListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      // check event type - sliders send structural events too!
      if (e.getPropertyName().compareTo("value") != 0) {
        return;
      }
      Slider slider = (Slider) e.getSource();
      int index = sliders.indexOf(slider);
      
      activeIndex = index;
      values.set(index, slider.getValue());
      setValueInternally(values);
      
      // inform listeners
      firePropertyChange("values", null, values);
    }
  }
  
  /**
   * Listener to NumericTextField events<br>
   *  sets values and slider and fires PropertyChange event
   */
  protected class MyTextFieldListener implements PropertyChangeListener {
    public void propertyChange(PropertyChangeEvent e) {
      // get values
      double val = textField.getValue();  // text field has new values
      
      // set new value, which may be adapted to bounds
      values.set(activeIndex, val);
      setValueInternally(values);
      
      firePropertyChange("values", null, new Double(val));
    }
  }
  
  protected class MyMouseListener extends MouseAdapter {
    public void mouseClicked(MouseEvent e) {
      Slider slider = (Slider) e.getSource();
      slider.requestFocusInWindow();
    }
  }
  
  /**
   * Listener to focus events.<br>
   * Changes background color and activeIndex
   */
  protected class MyFocusListener implements FocusListener {
    public void focusGained(FocusEvent e) {
      Slider slider = (Slider) e.getSource();
      int index = sliders.indexOf(slider);
      if (index == activeIndex) {     // no change
        return;
      }
      
      // restore old slider
      sliders.get(activeIndex).setMarked(false);
      
      // mark new slider
      activeIndex = index;
      slider.setMarked(true);
      textField.setValueInternally(values.get(activeIndex));
    }
    
    public void focusLost(FocusEvent e) {
    }
    
  }
}
