/*
 * CVector.java
 *
 * Created on 17. Januar 2003, 17:33
 */
package physbeans.math;

/**
 * vector of complex numbers
 */
public class CVector implements Cloneable {

  protected DVector re,  im;

  /** Creates a complex vector with n elements */
  public CVector(int n) {
    re = new DVector(n);
    im = new DVector(n);
  }

  /**
   * Creates a complex vector from DVectors of getReal and imaginary values.
   * The arrays have to have the same length.
   */
  public CVector(DVector re, DVector im) {
    this.re = re;
    this.im = im;
  }

  /**
   * Creates a complex vector from vectors of getReal and imaginary values.
   * The arrays have to have the same length.
   */
  public CVector(double[] r, double[] i) {
    this(new DVector(r), new DVector(i));
  }

  /**
   * Creates a complex vector from an array of getReal parts.
   * Imaginary parts are all 0.
   */
  public CVector(double[] re) {
    this.re = new DVector(re);
    im = new DVector(re.length);
  }

  /**
   * Creates a complex vector from a vector of getReal parts.
   */
  public CVector(DVector re) {
    this.re = re;
    im = new DVector(re.getDimension());
  }

  /**
   * Gets a single element of the vector
   */
  public Complex get(int i) {
    return new Complex(re.get(i), im.get(i));
  }

  /**
   * Sets a single element of the vector
   */
  public void set(int i, Complex z) {
    re.set(i, z.real());
    im.set(i, z.imag());
  }

  /**
   * Gets dimension of complex array.
   */
  public int getDimension() {
    return re.getDimension();
  }

  /**
   * Get vector of getReal parts
   */
  public DVector getReal() {
    return re;
  }

  /**
   * Sets vector of getReal parts.
   * The imaginary parts are unchanged.
   */
  public void setReal(DVector re) {
    if (re.getDimension() == this.re.getDimension()) {
      this.re = re;
    }
  }

  /**
   * Gets vector of imaginary parts.
   */
  public DVector getImag() {
    return im;
  }

  /**
   * Sets vector of imaginary parts.
   * The getReal parts are unchanged.
   */
  public void setImag(DVector im) {
    if (im.getDimension() == this.im.getDimension()) {
      this.im = im;
    }
  }

  /**
   * Gets vector of absolute values.
   */
  public DVector getAbs() {
    int n = re.getDimension();
    double[] r = new double[n];

    for (int i = 0; i < n; i++) {
      r[i] = Complex.abs(get(i));
    }

    return new DVector(r);
  }

  /**
   * Sets all absolute values.
   * The phase angles are unchanged.
   */
  public void setAbs(DVector r) {
    int n = re.getDimension();
    if (r.getDimension() == n) {
      DVector phase = getPhase();
      for (int i = 0; i < n; i++) {
        set(i, Complex.createFromPolar(r.get(i), phase.get(i)));
      }
    }
  }

  /**
   * Gets vector of getPhase values.
   */
  public DVector getPhase() {
    int n = re.getDimension();
    double[] phi = new double[n];

    for (int i = 0; i < n; i++) {
      phi[i] = Complex.argument(get(i));
    }

    return new DVector(phi);
  }

  /**
   * Sets all phase angles.
   * The absolute values are unchanged.
   */
  public void setPhase(DVector phi) {
    int n = re.getDimension();
    if (phi.getDimension() == n) {
      DVector r = getAbs();
      for (int i = 0; i < n; i++) {
        set(i, Complex.createFromPolar(r.get(i), phi.get(i)));
      }
    }
  }

  /**
   * Returns the norm of the complex vector.
   */
  public double norm() {
    return Math.hypot(re.norm(), im.norm());
  }

  /**
   * Multiply the vector by a scalar in place, A = s*A.
   */
  public CVector timesEquals(double s) {
    re.timesEquals(s);
    im.timesEquals(s);
    return this;
  }

  /**
   * returns a default String representation of the DVector.
   */
  public String toString() {
    String result = "[";
    int n = re.getDimension();
    for (int i = 0; i < n - 1; i++) {
      result += get(i).toString() + ",";
    }
    result += get(n - 1).toString() + "]";

    return result;
  }

  /**
   * returns a String representation of the DVector, using a given precision.
   * @param prec         precision of the doubles
   * @param withNewline  if true, String contains only one element per line
   */
  public String toString(int prec, boolean withNewline) {
    String result = "[";
    int n = re.getDimension();
    for (int i = 0; i < n - 1; i++) {
      result += get(i).toString(prec) + ",";
      if (withNewline) {
        result += "\n";
      }
    }
    result += get(n - 1).toString(prec) + "]";

    return result;
  }
}
