package physbeans.math;

import java.io.*;
import java.util.*;
import physbeans.model.*;

/**
 * Vector of doubles - one-dimensional version of Jama = Java Matrix class.
 */
public class DVector implements Cloneable, Serializable {

  /**
   * Array of vector values.
   */
  protected double[] v;
  static final long serialVersionUID = -3660843485213893345L;

  /**
   * Creates a 1-dim vector from a given double value.
   */
  public DVector(double v1) {
    this.v = new double[]{v1};
  }

  /**
   * Creates a 2-dim vector from two given double values.
   */
  public DVector(double v1, double v2) {
    this.v = new double[]{v1, v2};
  }

  /**
   * Creates a 3-dim vector from three given double values.
   */
  public DVector(double v1, double v2, double v3) {
    this.v = new double[]{v1, v2, v3};
  }

  /**
   * Creates a 4-dim vector from four given double values.
   */
  public DVector(double v1, double v2, double v3, double v4) {
    this.v = new double[]{v1, v2, v3, v4};
  }

  /**
   * Construct an n-dim vector of zeros
   *
   * @param n dimension of vector.
   */
  public DVector(int n) {
    v = new double[n];
  }

  /**
   * Construct a vector from a 1-D array without copying
   *
   * @param v array of doubles.
   */
  public DVector(double[] v) {
    this.v = v;
  }

  /**
   * Construct an n-dim vector filled with a given value.
   *
   * @param n Dimension of vector.
   * @param s Fill the vector with this scalar value.
   */
  static public DVector createFilledDVector(int n, double s) {
    double[] v = new double[n];
    for (int i = 0; i < n; i++) {
      v[i] = s;
    }
    return new DVector(v);
  }

  /**
   * Element-by-element division, C = A ./ B
   *
   * @param B another matrix
   * @return A ./ B
   */
  public DVector arrayDivide(DVector B) {
    checkVectorDimensions(B);
    DVector X = new DVector(v.length);
    double[] C = X.getArray();
    for (int i = 0; i < v.length; i++) {
      C[i] = v[i] / B.v[i];
    }
    return X;
  }

  /**
   * Element-by-element division in place, A = A ./ B
   *
   * @param B another Vector
   * @return A ./ B
   */
  public DVector arrayDivideEquals(DVector B) {
    checkVectorDimensions(B);
    for (int i = 0; i < v.length; i++) {
      v[i] = v[i] / B.v[i];
    }
    return this;
  }

  /**
   * Element-by-element multiplication, C = A.*B
   *
   * @param B another matrix
   * @return A.*B
   */
  public DVector arrayTimes(DVector B) {
    checkVectorDimensions(B);
    DVector X = new DVector(v.length);
    double[] C = X.getArray();
    for (int i = 0; i < v.length; i++) {
      C[i] = v[i] * B.v[i];
    }
    return X;
  }

  /**
   * Element-by-element multiplication in place, A = A.*B
   *
   * @param B another matrix
   * @return A.*B
   */
  public DVector arrayTimesEquals(DVector B) {
    checkVectorDimensions(B);
    for (int i = 0; i < v.length; i++) {
      v[i] = v[i] * B.v[i];
    }
    return this;
  }

  /**
   * Check if size(A) == size(B) *
   */
  private void checkVectorDimensions(DVector B) {
    if (B.getDimension() != v.length) {
      throw new IllegalArgumentException("Vector dimensions must agree.");
    }
  }

  /**
   * Clone the DVector object.
   */
  public Object clone() {
    return this.copy();
  }

  /**
   * Construct a vector from a copy of another one.
   *
   * @param v Original vector.
   * @exception IllegalArgumentException All rows must have the same length
   */
  public static DVector constructWithCopy(double[] v) {
    int n = v.length;
    DVector X = new DVector(n);
    double[] C = X.getArray();
    for (int i = 0; i < n; i++) {
      C[i] = v[i];
    }
    return X;
  }

  /**
   * Make a deep copy of a DVector
   */
  public DVector copy() {
    DVector X = new DVector(v.length);
    double[] C = X.getArray();
    for (int i = 0; i < v.length; i++) {
      C[i] = v[i];
    }
    return X;
  }

  /**
   * Get a single element.
   *
   * @param i Row index.
   * @return v[i]
   * @exception ArrayIndexOutOfBoundsException
   */
  public double get(int i) {
    return v[i];
  }

  /**
   * Access the internal one-dimensional array.
   *
   * @return Pointer to the one-dimensional array of vector elements.
   */
  public double[] getArray() {
    return v;
  }

  /**
   * Copy the internal one-dimensional array.
   *
   * @return one-dimensional array copy of matrix elements.
   */
  public double[] getArrayCopy() {
    double[] C = new double[v.length];
    for (int i = 0; i < v.length; i++) {
      C[i] = v[i];
    }
    return C;
  }

  /**
   * Get a subvector.
   *
   * @param i0 Initial index
   * @param i1 Final index
   * @return v(i0:i1)
   * @exception ArrayIndexOutOfBoundsException Submatrix indices
   */
  public DVector getDVector(int i0, int i1) {
    DVector X = new DVector(i1 - i0 + 1);
    double[] B = X.getArray();
    try {
      for (int i = i0; i <= i1; i++) {
        B[i - i0] = v[i];
      }
    } catch (ArrayIndexOutOfBoundsException e) {
      throw new ArrayIndexOutOfBoundsException("Submatrix indices");
    }
    return X;
  }

  /**
   * Get a subvector.
   *
   * @param r Array of row indices.
   * @return v(r(:))
   * @exception ArrayIndexOutOfBoundsException Submatrix indices
   */
  public DVector getDVector(int[] r) {
    DVector X = new DVector(r.length);
    double[] B = X.getArray();
    try {
      for (int i = 0; i < r.length; i++) {
        B[i] = v[r[i]];
      }
    } catch (ArrayIndexOutOfBoundsException e) {
      throw new ArrayIndexOutOfBoundsException("Submatrix indices");
    }
    return X;
  }

  /**
   * Gets a subvector given by an index range.
   *
   * @param idx
   * @return v(idx)
   */
  public DVector getDVector(IndexRange idx) {
    DVector X = new DVector(idx.getSize());
    double[] B = X.getArray();
    try {
      int i, j;
      if (idx.getStride() > 0) {
        for (i = idx.getStart(), j = 0; i <= idx.getEnd(); i += idx.getStride(), j++) {
          B[j] = v[i];
        }
      } else {
        for (i = idx.getStart(), j = 0; i >= idx.getEnd(); i += idx.getStride(), j++) {
          B[j] = v[i];
        }
      }
    } catch (ArrayIndexOutOfBoundsException e) {
      throw new ArrayIndexOutOfBoundsException("Submatrix indices");
    }
    return X;
  }

  /**
   * Get dimension i.e. length.
   *
   * @return m, the number of elements.
   */
  public int getDimension() {
    return v.length;
  }

  /**
   * C = A - B
   *
   * @param B another vector
   * @return A - B
   */
  public DVector minus(DVector B) {
    checkVectorDimensions(B);
    DVector X = new DVector(v.length);
    double[] C = X.getArray();
    for (int j = 0; j < v.length; j++) {
      C[j] = v[j] - B.v[j];
    }
    return X;
  }

  /**
   * A = A - B
   *
   * @param B another vector
   * @return A - B
   */
  public DVector minusEquals(DVector B) {
    checkVectorDimensions(B);
    for (int i = 0; i < v.length; i++) {
      v[i] = v[i] - B.v[i];
    }
    return this;
  }

  /**
   * norm
   *
   * @return sqrt of sum of squares of all elements.
   */
  public double norm() {
    double f = 0;
    for (int i = 0; i < v.length; i++) {
      f = Math.hypot(f, v[i]);
    }
    return f;
  }

  /**
   * C = A + B
   *
   * @param B another Vector
   * @return A + B
   */
  public DVector plus(DVector B) {
    checkVectorDimensions(B);
    DVector X = new DVector(v.length);
    double[] C = X.getArray();
    for (int i = 0; i < v.length; i++) {
      C[i] = v[i] + B.v[i];
    }
    return X;
  }

  /**
   * A = A + B. Changes the calling vector and returns the result.
   *
   * @param B another vector
   * @return A + B
   */
  public DVector plusEquals(DVector B) {
    checkVectorDimensions(B);
    for (int i = 0; i < v.length; i++) {
      v[i] = v[i] + B.v[i];
    }
    return this;
  }

  /**
   * Generate vector with random elements
   *
   * @param n Number of elements.
   * @return An n vector with uniformly distributed random elements.
   */
  public static DVector random(int n) {
    DVector A = new DVector(n);
    double[] X = A.getArray();
    for (int i = 0; i < n; i++) {
      X[i] = Math.random();
    }
    return A;
  }

  /**
   * Set a single element.
   *
   * @param i Row index.
   * @param s v[i].
   * @exception ArrayIndexOutOfBoundsException
   */
  public void set(int i, double s) {
    v[i] = s;
  }

  /**
   * Set a subvector.
   *
   * @param i0 Initial index
   * @param i1 Final index
   * @param X v[i0:i1]
   * @exception ArrayIndexOutOfBoundsException Submatrix indices
   */
  public void setDVector(int i0, int i1, DVector X) {
    try {
      for (int i = i0; i <= i1; i++) {
        v[i] = X.get(i - i0);
      }
    } catch (ArrayIndexOutOfBoundsException e) {
      throw new ArrayIndexOutOfBoundsException("Submatrix indices");
    }
  }

  /**
   * Set a subvector.
   *
   * @param r Array of indices.
   * @param X v(r(:))
   * @exception ArrayIndexOutOfBoundsException Submatrix indices
   */
  public void setDVector(int[] r, DVector X) {
    try {
      for (int i = 0; i < r.length; i++) {
        v[r[i]] = X.get(i);
      }
    } catch (ArrayIndexOutOfBoundsException e) {
      throw new ArrayIndexOutOfBoundsException("Submatrix indices");
    }
  }

  /**
   * Multiply a vector by a scalar, C = s*A
   *
   * @param s scalar
   * @return s*A
   */
  public DVector times(double s) {
    DVector X = new DVector(v.length);
    double[] C = X.getArray();
    for (int i = 0; i < v.length; i++) {
      C[i] = s * v[i];
    }
    return X;
  }

  /**
   * scalar product A * B
   *
   * @param B another vector
   * @return scalar product, A * B
   * @exception IllegalArgumentException Vector dimensions must agree.
   */
  public double times(DVector B) {
    if (B.getDimension() != v.length) {
      throw new IllegalArgumentException("Vector dimensions must agree.");
    }
    double sum = 0.0;
    for (int i = 0; i < v.length; i++) {
      sum += v[i] * B.v[i];
    }
    return sum;
  }

  /**
   * Multiply a vector by a scalar in place, A = s*A
   *
   * @param s scalar
   * @return replace A by s*A
   */
  public DVector timesEquals(double s) {
    for (int i = 0; i < v.length; i++) {
      v[i] = s * v[i];
    }
    return this;
  }

  /**
   * Multiply a vector by a matrix from left: result = M*v
   */
  public DVector times(Matrix m) {
    if (m.getColumnDimension() != v.length) {
      throw new IllegalArgumentException("Inner dimensions must agree.");
    }

    int rows = m.getRowDimension();
    double[] result = new double[rows];

    if (m.getColumnDimension() != v.length) {
      throw new IllegalArgumentException("Matrix must be square.");
    }

    for (int i = 0; i < rows; i++) {
      result[i] = 0.0;
      for (int j = 0; j < v.length; j++) {
        result[i] += m.get(i, j) * v[j];
      }
    }

    return new DVector(result);
  }

  /**
   * Unary minus
   *
   * @return -A
   */
  public DVector uminus() {
    DVector X = new DVector(v.length);
    double[] C = X.getArray();
    for (int i = 0; i < v.length; i++) {
      C[i] = -v[i];
    }
    return X;
  }

  /**
   * Reverses the sequence of elements.
   */
  public void reverse() {
    int size = v.length;
    double[] vRev = new double[size];
    for (int i = 0; i < size; i++) {
      vRev[i] = v[size - i - 1];
    }
    v = vRev;
  }

  /**
   * reads a String into a DVector
   */
  static public DVector valueOf(String s) {
    s = s.trim();
    s = s.substring(1, s.length() - 1);   // remove [,]
    String[] words = s.split("[,;]");
    int dim = words.length;

    /*
     // remove trailing empty string - workaround for libGDX problem
     if (words[0].equals("")) { 
     dim--;
     String[] temp = new String[dim];
     for (int i = 0; i<dim; i++) {
     temp[i] = words[i+1];
     }
     words = temp;
     }
     */
    
    DVector v = new DVector(dim);
    for (int i = 0; i < dim; i++) {
      String word = words[i].trim();
      try {
        v.set(i, Double.parseDouble(word));
      } catch (NumberFormatException e) {
        // check for 3 special cases
        switch (word) {
          case "infinity":
            v.set(i, Double.POSITIVE_INFINITY);
            break;
          case "-infinity":
            v.set(i, Double.NEGATIVE_INFINITY);
            break;
          case "NaN":
            v.set(i, Double.NaN);
            break;
          default:
            throw new IllegalArgumentException("wrong format in " + s);
        }
      }
    }

    return v;
  }

  /**
   * returns a String representation of the DVector
   */
  public String toString() {
    int n = v.length;
    if (n == 0) {
      return "[]";
    }

    String result = "[";
    for (int i = 0; i < n - 1; i++) {
      result += v[i] + ",";
    }
    result += v[n - 1] + "]";

    return result;
  }

  /**
   * returns a String representation of the DVector, using a given precision.
   *
   * @param prec precision of the doubles
   * @param withNewline if true, String contains only one element per line
   */
  public String toString(int prec, boolean withNewline) {
    String result = "[";
    int n = v.length;
    for (int i = 0; i < n - 1; i++) {
      result += Maths.toString(v[i], prec) + ",";
      if (withNewline) {
        result += "\n";
      }
    }
    result += Maths.toString(v[n - 1], prec) + "]";

    return result;
  }

  /**
   * Searches the DVector for the specified value using the binary search
   * algorithm. The vector must be sorted, otherwise the results are undefined.
   * If the array contains multiple elements with the specified value, there is
   * no guarantee which one will be found.
   *
   * @return index of the search key, if it is contained in the vector;
   * otherwise, (-(insertion point) - 1). The insertion point is defined as the
   * point at which the key would be inserted into the list: the index of the
   * first element greater than the key, or vector length, if the key is larger
   * than all vector elements.
   */
  public int binarySearch(double key) {
    return Arrays.binarySearch(v, key);
  }

  /**
   * Sorts the vector in ascending order.
   */
  public void sort() {
    Arrays.sort(v);
  }

  /**
   * Sorts the vector in ascending order and returns an int vector giving the
   * positions of the old elements in the sorted vector.
   */
  public int[] sortWithIndex() {
    DVector vOld = copy();
    Arrays.sort(v);

    int[] index = new int[v.length];
    for (int i = 0; i < v.length; i++) {
      for (int j = 0; j < v.length; j++) {
        if (vOld.get(i) == v[j]) {
          index[i] = j;
        }
      }
    }

    return index;
  }

  /**
   * Returns the first element of the vector.<p>
   * Conveniance method for vectors that are used as coordinates.
   */
  public double x() {
    return v[0];
  }

  /**
   * Returns the second element of the vector.<p>
   * Conveniance method for vectors that are used as coordinates.
   */
  public double y() {
    return v[1];
  }

  /**
   * Returns the third element of the vector.<p>
   * Conveniance method for vectors that are used as coordinates.
   */
  public double z() {
    return v[2];
  }

  /**
   * Creates a diagonal matrix from the vector.
   */
  public Matrix diag() {
    int n = v.length;
    Matrix D = new Matrix(n, n);

    for (int i = 0; i < n; i++) {
      D.set(i, i, v[i]);
    }
    return D;
  }

  /**
   * Returns largest element of the vector.
   */
  public double max() {
    double maxVal = Double.NEGATIVE_INFINITY;
    for (int i = 0; i < v.length; i++) {
      maxVal = Math.max(maxVal, v[i]);
    }
    return maxVal;
  }

  /**
   * Returns smallest element of the vector.
   */
  public double min() {
    double minVal = Double.POSITIVE_INFINITY;
    for (int i = 0; i < v.length; i++) {
      minVal = Math.min(minVal, v[i]);
    }
    return minVal;
  }

  /**
   * Returns vector with elements that are shifted by p elements, filling with
   * 0's. Positive shift means down/right.
   */
  public DVector shift(int p) {
    int l = v.length;
    DVector result = new DVector(l);

    if ((p >= 0) & (p < l)) {
      result.setDVector(p, l - 1, this.getDVector(0, l - p - 1));
    } else if ((p < 0) & (p > -l)) {
      result.setDVector(0, l + p - 1, this.getDVector(-p, l - 1));
    }  // else do nothing -> result contains only 0's

    return result;
  }

  /**
   * Returns vector with elements that are shifted one element to the left (or
   * up). The last element is set to 0.0.
   */
  public DVector shiftLeft() {
    return shift(-1);
  }

  /**
   * Returns vector with elements that are shifted one element to the right (or
   * down). The first element is set to 0.0.
   */
  public DVector shiftRight() {
    return shift(1);

  }

  /**
   * Creates an array of booleans from the vector.
   * <br>The result is true if and only if the corresponding vector element is >
   * 0.
   * <P>
   * Interface function to make doubles work as booleans.
   */
  public boolean[] toBoolean() {
    int n = v.length;
    boolean[] result = new boolean[n];
    for (int i = 0; i < n; i++) {
      result[i] = (v[i] > 0);
    }
    return result;
  }
}
