package physbeans.math;

import physbeans.model.*;

/**
 * Complex matrix implemented as 1d array for time critical computations.
 * Performes no checks for array sizes.
 * The internal vectors can be larger than the dimensions of the matrix
 * so that a FastCMatrix can be reused as fast workspace.
 */
public class FastCMatrix {

  protected double[] re,  im;
  protected int m,  n;

  /*
   * Creates a FastCMatrix with m rows and n columns.
   */
  public FastCMatrix(int m, int n) {
    re = new double[m * n];
    im = new double[m * n];
    this.m = m;
    this.n = n;
  }

  /*
   * Changes the dimensions of the FastCMatrix.
   * Allocates new workspace only if the size grows.
   */
  public void resize(int m1, int n1) {
    if (m1 * n1 > m * n) {
      re = new double[m1 * n1];
      im = new double[m1 * n1];
    }
    m = m1;
    n = n1;
  }

  /*
   * Creates a (small) FastCMatrix from a part of a (large) X defined by
   * IndexRange's in x and y direction.
   */
  public void setMatrix(IndexRange idx, IndexRange idy, CMatrix X) {
    // stores X by rows
    int idx0 = idx.getStart();
    int idxs = idx.getStride();
    int idx1 = idx.getEnd();
    int idy0 = idy.getStart();
    int idys = idy.getStride();
    int idy1 = idy.getEnd();

    int ij = 0;
    for (int i = idx0; i <= idx1; i += idxs) {
      for (int j = idy0; j <= idy1; j += idys) {
        re[ij] = X.re[i][j];
        im[ij] = X.im[i][j];
        ij++;
      }
    }
  }

  /*
   * Implements this = A .* B writing directly to the workspace of this.
   */
  public FastCMatrix setArrayTimes(FastCMatrix A, FastCMatrix B) {
    for (int i = 0; i < m * n; i++) {
      double reTemp = A.re[i] * B.re[i] - A.im[i] * B.im[i];
      im[i] = A.re[i] * B.im[i] + A.im[i] * B.re[i];
      re[i] = reTemp;
    }
    return this;
  }

  /*
   * Implements this = this .* B.
   */
  public FastCMatrix arrayTimesEquals(FastCMatrix B) {
    for (int i = 0; i < m * n; i++) {
      double reTemp = re[i] * B.re[i] - im[i] * B.im[i];
      im[i] = re[i] * B.im[i] + im[i] * B.re[i];
      re[i] = reTemp;
    }
    return this;
  }

  /*
   * Implements this = c * this.
   */
  public FastCMatrix timesEquals(Complex c) {
    double s = c.real();
    double t = c.imag();
    for (int i = 0; i < m * n; i++) {
      double reTemp = s * re[i] - t * im[i];
      im[i] = t * re[i] + s * im[i];
      re[i] = reTemp;
    }
    return this;
  }

  /*
   * Implements this = this + B.
   */
  public FastCMatrix plusEquals(FastCMatrix B) {
    for (int i = 0; i < m * n; i++) {
      re[i] += B.re[i];
      im[i] += B.im[i];
    }
    return this;
  }
}
