package physbeans.math;

/**
 * Class performing interpolation over a two dimensional grid.<P>
 * Only method implemented so far: bilinear interpolation over an equally
 * spaced grid.
 */
public class Interpolator2D {
  
  double[][] grid;
  protected double xMin, xMax, yMin, yMax;
  protected int bx, by;
  
  /**
   * Creates a new Interpolator2D for a given grid of function values and
   * intervals of x and y values corresponding to the smallest/largest index
   * values.
   */
  public Interpolator2D(double[][] grid, double xMin, double xMax,
      double yMin, double yMax) {
    this.grid = grid;
    this.xMin = xMin;
    this.xMax = xMax;
    this.yMin = yMin;
    this.yMax = yMax;
    bx = 0;
    by = 0;
  }
  
  /**
   * Creates a new Interpolator2D for a given grid of function values with
   * boundaries of given size. The given x and y values correspond to
   * the smallest/largest index values of the inner part of the grid, e.g.
   *     grid[by][bx] -> (xmin, ymin)
   */
  public Interpolator2D(double[][] grid, int bx, int by,
      double xMin, double xMax, double yMin, double yMax) {
    this.grid = grid;
    this.xMin = xMin;
    this.xMax = xMax;
    this.yMin = yMin;
    this.yMax = yMax;
    this.bx = bx;
    this.by = by;
  }
  
  /**
   * Provides a new grid for the interpolator.
   */
  public void setGrid(double[][] g) {
    grid = g;
  }
  
  /**
   * Computes the function value at (x,y) computed by bilinear interpolation.
   * Returns NaN for points outside the grid.
   */
  public double getValue(double x, double y) {
    int N = grid[0].length - 2*bx;
    int M = grid.length - 2*by;
    
    double j = (x - xMin)/(xMax - xMin) * (N-1) + bx;
    if (Maths.isStrictlyPositive(bx - j) || 
        Maths.isStrictlyPositive(j - N - bx + 1)) {
      return Double.NaN;
    }
    
    double i = (yMax - y)/(yMax - yMin) * (M-1) + by;
    if (Maths.isStrictlyPositive(by - i) ||
        Maths.isStrictlyPositive(i - M - by + 1)) {
      return Double.NaN;
    }
    
    int i0 = (int) Math.floor(i + Maths.getAccuracy());
    int j0 = (int) Math.floor(j + Maths.getAccuracy());
    double s = i - i0;
    double t = j - j0;
    
    if ((i0 == M-1+by) && (j0 == N-1+bx)) {
      return grid[i0][j0];
    } else if (i0 == M-1+by) {
      return (1-t)*grid[i0][j0] +  t*grid[i0][j0+1];
    } else if (j0 == N-1+bx) {
      return (1-s)*grid[i0][j0] +  s*grid[i0+1][j0];
    } else {
      return (1-s)*(1-t)*grid[i0][j0] + s*(1-t)*grid[i0+1][j0]
          + s*t*grid[i0+1][j0+1] + (1-s)*t*grid[i0][j0+1];
    }
  }
  
}
