package physbeans.math;

/**
 * adaptive solver that uses a special strategy to get a new step size.<br>
 * <br>
 * basic strategy: <br>
 *     integrate with actual stepsize in one step<br>
 *     integrate with half of actual stepsize in two steps<br>
 *     estimate the error to get new stepsize
 */
public class ODEAdaptiveSolver extends ODESolver {

  final protected double DEFAULT_TOLERANCE = 1.0e-6;
  protected double tol;         // max. error
  protected double safety;      // safety factor for step size
  protected double maxFac;      // maximal expansion factor of step size
  protected boolean throwStepsizeTooSmall;
  protected ODESingleStepSolver subSolver;
  protected int order;          // order of solver = order of ode1
  protected double tEnd;        // end point of integration
  protected double stepSize;    // current value of step size
  protected double gamma;       // factor for error estimation

  /**
   * construct adaptive solver for a given ODE defining<br>
   *   maximal local error<br>
   *   use Runge-Kutta-4 solver as single step solver
   */
  public ODEAdaptiveSolver(ODE ode) {
    this(ode, new ODESolverRK4(ode));
  }

  /**
   * construct adaptive solver for a given ODE defining<br>
   *   end time of integration<br>
   *   maximal local error<br>
   *   a single step solver for the integration in one and two half steps
   */
  public ODEAdaptiveSolver(ODE ode, ODESingleStepSolver solv) {
    super(ode); // sets t, x to initial values
    this.order = solv.order;
    this.subSolver = solv;
    this.tol = DEFAULT_TOLERANCE;
    safety = 0.95;
    maxFac = 5.0;
    gamma = 1.0 / (1.0 - Math.pow(2.0, -order));
    throwStepsizeTooSmall = true;
  }

  /**
   * proceed one step, using an appropriate step size
   */
  public void nextStep() throws StepsizeTooSmallException {
    boolean stepAccepted = false;
    while (!stepAccepted) {
      // solve in one large step
      subSolver.t = t;
      subSolver.x = x.copy();
      subSolver.nextStep(stepSize);
      DVector xOneStep = subSolver.x;

      // solve in two small steps
      subSolver.t = t;
      subSolver.x = x.copy();
      subSolver.nextStep(stepSize / 2.0);
      subSolver.nextStep(stepSize / 2.0);

      // estimate error
      // diff = gamma * (x2 - x1)
      DVector diff = subSolver.x.minus(xOneStep).timesEquals(gamma);
      double error = diff.norm();

      // compute new stepsize
      double hRel = Math.pow(tol / error, 1.0 / (1.0 + order));
      double stepSizeNew = stepSize * hRel;

      // throw exception, if step size gets too small
      if (throwStepsizeTooSmall && (Math.abs(stepSizeNew) < tol)) {
        throw new StepsizeTooSmallException("Step size " + stepSizeNew +
                " is smaller than tolerance " + tol);
      }
      if (hRel < 1) {
        // step rejected, use smaller step size
        stepSize = safety * stepSizeNew;
      } else {
        // step accepted, use new values
        stepAccepted = true;
        t += stepSize;
        x = xOneStep.plus(diff);

        // increase step size
        if (hRel > maxFac) { // don't expand too much !
          stepSize *= maxFac;
        } else {
          stepSize = safety * stepSizeNew;
        }
        stepSize = Math.min(stepSize, tEnd - t); // don't overshoot!
      }
    }
  }

  /**
   * integrate until t+dt<br>
   * return number of steps needed
   */
  public int nextStep(double dt) throws StepsizeTooSmallException {
    // old stepSize could be much too small while approaching tEnd from last step!
    // therefore: try with one step
    stepSize = dt;
    int nSteps = 0;
    tEnd = t + dt;
    while (t < tEnd - tol) {
      nextStep();
      nSteps++;
    }
    return nSteps;
  }

  /**
   * sets absolute tolerance<br>
   * default usually: 1e-6
   */
  public void setAbsoluteTolerance(double t) {
    tol = t;
  }

  /**
   * gets absolute tolerance
   */
  public double getAbsoluteTolerance() {
    return tol;
  }

  /**
   * sets maximal factor m<br>
   * new stepsize is smaller than m *old_stepsize <br>
   * should be larger than 1
   */
  public void setMaxFac(double m) {
    maxFac = m;
  }

  /**
   * gets maximal factor m
   */
  public double getMaxFac() {
    return maxFac;
  }

  /**
   * sets safety factor s<br>
   * multiplies proposed new stepsize<br>
   * should be smaller than 1 <br>
   * usually approx. 1
   */
  public void setSafety(double s) {
    safety = s;
  }

  /**
   * Gets safety factor s
   */
  public double getSafety() {
    return safety;
  }

  /**
   * Gets value of throwStepsizeTooSmall flag<br>
   * If set to true (default) the solver throws an exception when the stepsize
   * gets too small. In rare cases a sudden parameter change can lead to a few
   * very tiny steps, which would block further execution.
   * Set the flag to true there.
   */
  public boolean isThrowStepsizeTooSmall() {
    return throwStepsizeTooSmall;
  }

  /**
   * Sets value of throwStepsizeTooSmall flag
   */
  public void setThrowStepsizeTooSmall(boolean flag) {
     throwStepsizeTooSmall = flag;
  }

  /**
   * Sets ode to be solved
   */
  public void setOde(ODE ode) {
    // overwrites parent method to set ode of subsolver
    this.ode = ode;
    subSolver.setOde(ode);
  }
}
