package physbeans.phys;

import physbeans.math.*;
import java.awt.*;

/**
 * Sets initial values for eigen modes of a circular membrane.<p>
 * An eigen mode is defined by two integers (nr, nphi) giving the number of
 * maxima in radial and angular direction.
 * The size and position of the circle can be defined arbitrarily. If the
 * RippleTank provides corresponding circular walls, the eigen modes will result.
 * If not, the given initial values will disperse.
 */
public class CircularModeWaveSource extends InitializingWaveSource {
  
  protected int m, k;
  protected DVector vc;
  protected double r0;
  protected double a;
  
  /** Creates a new instance of RectangularModeWaveSource */
  public CircularModeWaveSource() {
    super();
    m = 2;
    k = 3;
    vc = new DVector(0.0, 0.0);
    r0 = 1.0;
    a = 1.0;
  }
  
  protected void setGrid(RippleTank tank) {
    double[][] grid = tank.getGrid();
    int sx = tank.getGridSizeX();
    int sy = tank.getGridSizeY();
    
    Point pc = tank.getIndex(vc);
    int pr0 = tank.getIndexForX(vc.x() + r0) - pc.x;   // r0 in pixel
    
    double mu = Bessel.besselJZero(m, k);
    double laRoot = mu/pr0;
    
    double r, phi;
    for (int x = 0; x < sx; x++) {
      for (int y = 0; y < sy; y++) {
        r = Math.hypot(x - pc.x, y - pc.y);
        if (r > pr0) continue;
        phi = Math.atan2(y - pc.y, x - pc.x);
        grid[y][x] = a*Math.cos(m*phi)*Bessel.jn(m, laRoot*r);
      }
    }
  }
  
  /**
   * Gets the number of the mode in phi direction.<p>
   * It corresponds to the index of the Bessel function.
   */
  public int getModeNumberPhi() {
    return m;
  }
  
  /**
   * Sets the number of the mode in phi direction.<p>
   * It corresponds to the index of the Bessel function.
   */
  public void setModeNumberPhi(int m) {
    this.m = m;
  }
  
  /**
   * Gets the number of the mode in radial direction.<p>
   * It corresponds to the number of the zero of the Bessel function.
   */
  public int getModeNumberR() {
    return k;
  }
  
  /**
   * Sets the number of the mode in radial direction.<p>
   * It corresponds to the number of the zero of the Bessel function.
   */
  public void setModeNumberR(int n) {
    k = n;
  }
  
  /**
   * Gets the coordinates of the center point.
   */
  public DVector getCenter() {
    return vc;
  }
  
  /**
   * Sets the coordinates of the center point.
   */
  public void setCenter(DVector v) {
    vc = v;
  }
  
  /**
   * Gets the radius of the circle.
   */
  public double getRadius() {
    return r0;
  }
  
  /**
   * Sets the radius of the circle.
   */
  public void setRadius(double r) {
    r0 = r;
  }
  
  /**
   * Gets the amplitude of the eigen mode.
   */
  public double getAmplitude() {
    return a;
  }
  
  /**
   * Sets the amplitude of the eigen mode.
   */
  public void setAmplitude(double aNew) {
    a = aNew;
  }
}
