package physbeans.phys;

import physbeans.math.*;
import physbeans.model.*;

/**
 * Physical Bean modelling a set of static electric point charges in 2d.
 * Main properties are number, individual charges and positions.
 * Computes a vector field (electric field strength)
 *   and two scalar fields (abs. value of field strength, electric potential).
 * Sends a generic trigger event to notify of general changes
 *   and a vector event with the field strength at a given point.
 */
public class ElectrostaticPointCharges extends VectorField {
  
  protected DVector charges;
  protected Point2dVector coords;
  
  protected double chargeScale;
  protected double distanceScale;
  protected boolean scalarfieldIsPotential;
  
  /** Creates a default set of point charges */
  public ElectrostaticPointCharges() {
    super(2,2);   // space is 2d, vectors are 2d
    charges = new DVector(1.0*Constants.E0, -1.0*Constants.E0);
    coords = new Point2dVector(new double[]{-1.0e-4, 1.0e-4},
    new double[]{0.0, 0.0});
    chargeScale = 1.0;
    distanceScale = 1.0;
    scalarfieldIsPotential = false;
  }
  
  /**
   * Gets electric field strength at given coordinates v.
   */
  public DVector getVectorValue(DVector v) {
    DVector eField = new DVector(2);
    
    for (int i = 0; i < charges.getDimension(); i++) {
      DVector scaledPosition = coords.getPoint(i).times(distanceScale);
      DVector diff = v.minus(scaledPosition);
      double r = diff.norm();
      diff.timesEquals(chargeScale*charges.get(i)/(r*r*r));
      eField.plusEquals(diff);
    }
    
    eField.timesEquals(1.0/(4*Math.PI*Constants.EPSILON0));
    return eField;
  }
  
  /**
   * Returns a scalar value associated with the vector field
   *   at the given coordinates.<br>
   * This is either the absolute value or the field 's potential depending
   * on the value of flag scalarfieldIsPotential.
   */
  public double getScalarValue(DVector v) {
    if (scalarfieldIsPotential) {
      return getPotential(v);
    } else {
      return getAbsoluteValue(v);
    }
  }
  
  /**
   * Returns the absolute value at the given coordinates.<br>
   */
  public double getAbsoluteValue(DVector v) {
    return getVectorValue(v).norm();
  }
  
  /**
   * Gets electric potential at given coordinates v.
   */
  public double getPotential(DVector v) {
    double potential = 0.0;
    
    for (int i = 0; i < charges.getDimension(); i++) {
      DVector scaledPosition = coords.getPoint(i).times(distanceScale);
      DVector diff = v.minus(scaledPosition);
      double r = diff.norm();
      potential += chargeScale*charges.get(i)/r;
    }
    
    potential /= 4*Math.PI*Constants.EPSILON0;
    return potential;
  }
  
  /**
   * Gets vector with all charge values.
   */
  public DVector getCharges() {
    return charges;
  }
  
  /**
   * Sets all charge values.
   */
  public void setCharges(DVector q) {
    if (q.getDimension() == charges.getDimension()) {
      charges = q;
      trigger();
    }
  }
  
  /**
   * Gets value of charge i.
   */
  public double getCharge(int i) {
    return charges.get(i);
  }
  
  /**
   * Sets value of charge i.
   */
  public void setCharge(int i, double charge) {
    if (i >= 0 && i < charges.getDimension()) {
      charges.set(i, charge);
      trigger();
    }
  }
  
  /**
   * Gets number of charges.
   */
  public int getNCharges() {
    return charges.getDimension();
  }
  
  /**
   * Sets number of charges.
   * Charges all have a default value of 1e-10 [C] and are positioned along the
   * x axis with equal distance of 1 [m].
   */
  public void setNCharges(int nCharges) {
    charges = DVector.createFilledDVector(nCharges, 1.0e-10);
    coords = new Point2dVector(nCharges);
    trigger();
  }
  
  /**
   * Gets coordinates of all charges.
   */
  public Point2dVector getCoordinates() {
    return coords;
  }
  
  /**
   * Sets coordinates of all charges.
   */
  public void setCoordinates(Point2dVector coordinates) {
    if (coordinates.size() == coords.size()) {
      coords = coordinates;
      trigger();
    }
  }
  
  /**
   * Gets position vector of charge i.
   */
  public DVector getCoordinate(int i) {
    return coords.getPoint(i);
  }
  
  /**
   * Sets position vector of charge i.
   */
  public void setCoordinate(int i, DVector position) {
    if (i >= 0 && i < coords.size()) {
      coords.setPoint(i, position);
      trigger();
    }
  }
  
  /**
   * Gets scale factor of charges.
   */
  public double getChargeScale() {
    return this.chargeScale;
  }
  
  /**
   * Sets scale factor of charges.
   * This allows to scale the value of all charges by a common factor.
   */
  public void setChargeScale(double chargeScale) {
    this.chargeScale = chargeScale;
    trigger();
  }
  
  /**
   * Gets scale factor of distances.
   */
  public double getDistanceScale() {
    return this.distanceScale;
  }
  
  /**
   * Sets scale factor of distances.
   * This allows to scale the value of all distances by a common factor.
   */
  public void setDistanceScale(double distanceScale) {
    this.distanceScale = distanceScale;
    trigger();
  }
  
  /**
   * Gets value of flag scalarfieldIsPotential.
   * If true, the associated scalarfeld is the potential of the vector field,
   * otherwise it is its absolute value.
   */
  public boolean getScalarfieldIsPotential() {
    return this.scalarfieldIsPotential;
  }
  
  /**
   * Sets value of flag scalarfieldIsPotential.
   * If true, the associated scalarfeld is the potential of the vector field,
   * otherwise it is its absolute value.
   */
  public void setScalarfieldIsPotential(boolean scalarfieldIsPotential) {
    this.scalarfieldIsPotential = scalarfieldIsPotential;
    trigger();
  }
  
}
