package physbeans.phys;

import physbeans.math.*;

/**
 * A massive glass ball of given radius and index of refraction.
 * The reference point for its position is its center. The outer medium is
 * assumed to have an index of refraction equal to 1.
 */
public class GlassBall extends OpticalElement {
  
  protected double n;
  protected double r;
  
  /** Creates a new instance of GlassBall */
  public GlassBall() {
    n = 1.459;
    r = 5.0;
  }
  
  protected void extend(LightRay ray) {
    DVector pLast = ray.getLastPoint();
    double al = ray.getAngle();
    
    Circle2D circle = new Circle2D(new DVector(pos, 0.0), r);
    Ray2D endRay = new Ray2D(pLast, al);
    DVector pHit = circle.getRayHitPoint(endRay);
    if (Double.isNaN(pHit.x())) {
      return;    // miss
    }
    ray.addPoint(pHit);
    
    DVector inVec = new DVector(Math.cos(al), Math.sin(al));
    DVector nVec = circle.getNormal(pHit);
    double ni = 1.0;
    double no = n;
    if (Maths.isStrictlyPositive(inVec.times(nVec))) {
      ni = n;
      no = 1.0;
    }
    ray.setAngle(OpticalSurface.getRefractionAngle(inVec, nVec, ni, no));
    propagate(ray);
  }
  
  /**
   * Gets the index of refraction.
   */
  public double getIndexOfRefraction() {
    return n;
  }
  
  /**
   * Sets the index of refraction.
   */
  public void setIndexOfRefraction(double n) {
    this.n = n;
    trigger();
  }
  
  /**
   * Gets the radius of the glass ball.
   */
  public double getRadius() {
    return r;
  }
  
  /**
   * Sets the radius of the glass ball.
   */
  public void setRadius(double r) {
    if (r > 0.0) {
      this.r = r;
      trigger();
    }
  }
}
