package physbeans.phys;

import physbeans.math.*;

/**
 * A glass plane of given width and index of refraction.
 * The reference point for its position is in the middle of
 * the two surfaces.<p>
 * Usually the index of refraction of the surrounding medium is set to 1.0,
 * but it can be set to different values on both sides.<p>
 * The plane can be horizontal. In that case the position is interpreted as a
 * y coordinate.
 */
public class GlassPlane extends OpticalElement {
  
  protected double n, nl, nr;
  protected boolean horizontal;
  protected double width;
  
  /** Creates a new instance of GlassPlane */
  public GlassPlane() {
    super();
    n = 1.459;
    nl = 1.0;
    nr = 1.0;
    width = 0.5;
    horizontal = false;
  }
  
  /**
   * Gets the index of refraction.
   */
  public double getIndexOfRefraction() {
    return n;
  }
  
  /**
   * Sets the index of refraction.
   */
  public void setIndexOfRefraction(double n) {
    this.n = n;
    trigger();
  }
  
  /**
   * Gets the index of refraction of the medium to the left of the glass plane.
   */
  public double getIndexOfRefractionLeft() {
    return nl;
  }
  
  /**
   * Sets the index of refraction of the medium to the left of the glass plane.
   */
  public void setIndexOfRefractionLeft(double nl) {
    this.nl = nl;
    trigger();
  }
  
  /**
   * Gets the index of refraction of the medium to the right of the glass plane.
   */
  public double getIndexOfRefractionRight() {
    return nr;
  }
  
  /**
   * Sets the index of refraction of the medium to the right of the glass plane.
   */
  public void setIndexOfRefractionRight(double nr) {
    this.nr = nr;
    trigger();
  }
  
  /**
   * Gets the width of the glass plane.
   */
  public double getWidth() {
    return this.width;
  }
  
  /**
   * Sets the width of the glass plane.
   */
  public void setWidth(double width) {
    if (width >= 0.0) {
      this.width = width;
      trigger();
    }
  }
  
  /**
   * Gets the horizontal flag:
   * true, if the plane is horizontal, false, if it is vertical.
   */
  public boolean isHorizontal() {
    return this.horizontal;
  }
  
  /**
   * Sets the horizontal flag:
   * true, if the plane is horizontal, false, if it is vertical.
   */
  public void setHorizontal(boolean horizontal) {
    this.horizontal = horizontal;
  }
  
  /**
   * Adds additional points and/or changes the final angle of the ray as it
   * passes through the glass plane.<P>
   * If the maximal number of points is reached, it does nothing.
   */
  protected void extend(LightRay ray) {
    if (horizontal) {
      propagateH(ray);
    } else {
      propagateV(ray);
    }
  }
  
  protected void propagateV(LightRay ray) {
    // uses recursion to propagate through both sides, including possible
    // total reflections
    
    double x = ray.getLastPoint().get(0);
    double al = ray.getAngle();
    double x1 = pos - width/2;
    double x2 = pos + width/2;
    
    if (Math.abs(al) < Math.PI/2) {   // ray goes to the right
      if (x < x1) {     // crossing left plane
        propagateThroughPlaneV(ray, x1, nl, n);
        propagate(ray);
      } else if (x < x2) {  // crossing right plane
        propagateThroughPlaneV(ray, x2, n, nr);
        propagate(ray);
      }
    } else if (Math.abs(al) > Math.PI/2) { // ray goes to the left
      if (x > x2) {     // crossing right plane
        propagateThroughPlaneV(ray, x2, nr, n);
        propagate(ray);
      } else if (x > x1) {  // crossing left plane
        propagateThroughPlaneV(ray, x1, n, nl);
        propagate(ray);
      }
    }
  }
  
  protected void propagateH(LightRay ray) {
    // uses recursion to propagate through both sides, including possible
    // total reflections
    
    double y = ray.getLastPoint().get(1);
    double al = ray.getAngle();
    double y1 = pos - width/2;
    double y2 = pos + width/2;
    
    if (al > 0.0) {   // ray goes upwards
      if (y < y1) {     // crossing lower plane
        propagateThroughPlaneH(ray, y1, nl, n);
        propagate(ray);
      } else if (y < y2) {  // crossing upper plane
        propagateThroughPlaneH(ray, y2, n, nr);
        propagate(ray);
      }
    } else if (al < 0.0) { // ray goes downwards
      if (y > y2) {     // crossing upper plane
        propagateThroughPlaneH(ray, y2, nr, n);
        propagate(ray);
      } else if (y > y1) {  // crossing lower plane
        propagateThroughPlaneH(ray, y1, n, nl);
        propagate(ray);
      }
    }
  }
  
  /**
   * Computes the propagation of a ray crossing a vertical plane at xs.
   * Indices of refraction are ni on the incoming side, no on the outgoing side.
   * Assumes that an intersection exists.
   */
  protected void propagateThroughPlaneV(LightRay ray, double xs, double ni, double no) {
    DVector p = ray.getLastPoint();
    double al = ray.getAngle();
    double y1 = p.get(1) + (xs - p.get(0))*Math.tan(al);
    ray.addPoint(xs, y1);
    
    if (width == 0.0) {
      return;    // just one plane -> angle doesn't change!
    }
    
    double sal1 = ni/no * Math.sin(al);
    if (Math.abs(sal1) > 1) {            // total reflection
      ray.setAngle(Math.PI - al);
    } else {                             // refraction
      double al1 = Math.asin(sal1);
      al1 = (al > Math.PI/2)? Math.PI-al1 : al1;    // careful with asin!
      al1 = (al < -Math.PI/2)? -Math.PI-al1 : al1;    // careful with asin!
      ray.setAngle(al1);
    }
  }
  
  /**
   * Computes the propagation of a ray crossing a horizontal plane at ys.
   * Indices of refraction are ni on the incoming side, no on the outgoing side.
   * Assumes that an intersection exists.
   */
  protected void propagateThroughPlaneH(LightRay ray, double ys, double ni, double no) {
    DVector p = ray.getLastPoint();
    double al = ray.getAngle();
    double x1 = p.get(0) + (ys - p.get(1))/Math.tan(al);
    ray.addPoint(x1, ys);
    
    if (width == 0.0) {
      return;    // just one plane -> angle doesn't change!
    }
    
    double sal1 = ni/no * Math.cos(al);
    if (Math.abs(sal1) > 1) {            // total reflection
      ray.setAngle(-al);
    } else {                             // refraction
      double al1 = Math.PI/2 - Math.asin(sal1);
      al1 = (al < 0.0)? -al1 : al1;    // careful with asin!
      ray.setAngle(al1);
    }
  }
}
