package physbeans.phys;

import physbeans.model.*;
import physbeans.math.*;
import java.awt.*;

/**
 * Scalar field that is defined on a two dimensional spacial grid.
 * Values at arbitrary inner points are interpolated, values outside return NaN.
 * <p> The outer elements of the grid are generally used as boundary, which has
 * predefined values or implements an absorption layer etc. This boundary is not
 * part of the "real world", it should not be displayed. Its size is defined
 * by boundaryWidthX, boundaryWidthY.
 * <p>If the square flag is set, the number of grid points ny is computed in
 * such a way that the grid spacing in x and y direction are equal.
 */
public abstract class Grid2DScalarField extends ScalarField
        implements GridWithBoundary {

  protected int nx,  ny;
  protected double val[][];    // attention: first index is y direction!
  protected double xMin,  xMax,  yMin,  yMax;
  protected int bwX,  bwY;
  protected boolean square;
  protected Interpolator2D interp;
  protected final static int NMAX = 10000;   // max. grid size

  /**
   * Creates a new instance of Grid2DScalarField
   */
  public Grid2DScalarField() {
    super();
    nx = 10;
    ny = 10;
    xMin = -1.0;
    xMax = 1.0;
    yMin = -1.0;
    yMax = 1.0;
    bwX = 1;
    bwY = 1;
    square = false;
  }

  /**
   * Computes the grid values.
   */
  abstract protected void computeGrid();

  /**
   * Initialises all grid structures and sets initial values.
   */
  protected void setupGrid() {
    if (square) {
      ny = (int) Math.round((yMax - yMin) / getDX()) + 2 * bwY + 1;

      // During initialisation of an applet very large  (or negative due to
      // integer overflow) intermediate grid sizes may appear when the
      // dimensions are very small and only x is set yet.
      if ((ny <= 0) || (ny > NMAX)) {
        ny = nx;
      }
    }

    if ((val == null) || (val[0].length != nx) || (val.length != ny)) {
      val = new double[ny][nx];
    }
    interp = new Interpolator2D(val, bwX, bwY, xMin, xMax, yMin, yMax);
  }

  /**
   * Initialises all grid structures including the grid values.
   */
  protected void initGrid() {
    setupGrid();
    computeGrid();
  }

  /**
   * Recomputes the grid and informs listeners.
   */
  protected void updateGrid() {
    computeGrid();
    trigger();
  }

  /**
   * Returns field value using bilinear interpolation of grid points.
   */
  public double getScalarValue(DVector v) {
    return interp.getValue(v.x(), v.y());
  }

  /**
   * Gets grid size in x direction in pixel.
   */
  public int getGridSizeX() {
    return nx;
  }

  /**
   * Sets grid size in x direction in pixel.
   */
  public void setGridSizeX(int n) {
    nx = n;
    setupGrid();
    trigger();
  }

  /**
   * Gets grid size in y direction in pixel.
   */
  public int getGridSizeY() {
    return ny;
  }

  /**
   * Sets grid size in y direction in pixel.
   * (only if grid is not square)
   */
  public void setGridSizeY(int m) {
    if (!square) {
      ny = m;
      setupGrid();
      trigger();
    }
  }

  /**
   * Gets x position of first grid column.
   */
  public double getXMin() {
    return xMin;
  }

  /**
   * Sets x position of first grid column.
   */
  public void setXMin(double x) {
    xMin = x;
    setupGrid();
    trigger();
  }

  /**
   * Gets x position of last grid column.
   */
  public double getXMax() {
    return xMax;
  }

  /**
   * Sets x position of last grid column.
   */
  public void setXMax(double x) {
    xMax = x;
    setupGrid();
    trigger();
  }

  /**
   * Gets y position of last grid row.
   */
  public double getYMin() {
    return yMin;
  }

  /**
   * Sets y position of last grid row.
   */
  public void setYMin(double y) {
    yMin = y;
    setupGrid();
    trigger();
  }

  /**
   * Gets y position of first grid row.
   */
  public double getYMax() {
    return yMax;
  }

  /**
   * Sets y position of first grid row.
   */
  public void setYMax(double y) {
    yMax = y;
    setupGrid();
    trigger();
  }

  /**
   * Gets vector (xMin, yMin).
   */
  public DVector getMinWorld() {
    return new DVector(xMin, yMin);
  }

  /**
   * Gets vector (xMax, yMax).
   */
  public DVector getMaxWorld() {
    return new DVector(xMax, yMax);
  }

  /**
   * Gets number of boundary columns (on one side).
   */
  public int getBoundaryWidthX() {
    return bwX;
  }

  /**
   * Sets number of boundary columns (on one side).
   */
  public void setBoundaryWidthX(int bw) {
    bwX = bw;
    setupGrid();
    trigger();
  }

  /**
   * Gets number of boundary rows (on one side).
   */
  public int getBoundaryWidthY() {
    return bwY;
  }

  /**
   * Sets number of boundary rows (on one side).
   */
  public void setBoundaryWidthY(int bw) {
    bwY = bw;
    setupGrid();
    trigger();
  }

  /**
   * Gets the grid with all computed scalar field values.
   */
  public double[][] getGrid() {
    return val;
  }

  /**
   * Computes the nearest pair of index values corresponding to position v.
   * <p>If a coordinate lies outside the grid, the corresponding index has
   * the value -1.
   */
  public Point getIndex(DVector v) {
    return new Point(getIndexForX(v.x()), getIndexForY(v.y()));
  }

  /**
   * Computes the nearest index corresponding to position x.<br>
   * Returns -1, if x lies outside the physical part of the grid.
   */
  public int getIndexForX(double x) {
    int px = (int) Math.round((x - xMin) / (xMax - xMin) * (nx - 2 * bwX - 1) + bwX);
    if ((px < bwX) || (px >= nx - bwX)) {
      return -1;
    } else {
      return px;
    }
  }

  /**
   * Computes the nearest index corresponding to position y.<br>
   * Returns -1, if y lies outside the physical part of the grid.
   */
  public int getIndexForY(double y) {
    int py = (int) Math.round((y - yMax) / (yMin - yMax) * (ny - 2 * bwY - 1) + bwY);
    if ((py < bwY) || (py >= ny - bwY)) {
      return -1;
    } else {
      return py;
    }
  }

  /**
   * Gets the world coordinates corresponding to given grid indices (i,j).
   */
  public DVector getWorldFromIndex(int i, int j) {
    double x, y;

    if ((i < bwX) || (i > nx - bwX - 1)) {
      x = Double.NaN;
    } else {
      x = xMin + (i - bwX) * getDX();
    }

    if ((j < bwY) || (j > ny - bwY - 1)) {
      y = Double.NaN;
    } else {
      y = yMax - (j - bwY) * getDY();
    }

    return new DVector(x, y);
  }

  /**
   * Returns the grid increment in x direction.
   */
  protected double getDX() {
    return (xMax - xMin) / (nx - 2 * bwX - 1);
  }

  /**
   * Returns the grid increment in y direction.
   * This is a positive value (by definition) even though larger grid indices
   * correspond to smaller y values.
   */
  protected double getDY() {
    return (yMax - yMin) / (ny - 2 * bwY - 1);
  }

  /**
   * Gets the square flag.<br>
   * If true, the number of grid points in y direction is computed automatically
   * to make the grid increments in x and y direction equal.
   */
  public boolean isSquare() {
    return square;
  }

  /**
   * Sets the square flag.<br>
   * If true, the number of grid points in y direction is computed automatically
   * to make the grid increments in x and y direction equal.
   */
  public void setSquare(boolean b) {
    square = b;
    setupGrid();
    trigger();
  }
}
