package physbeans.phys;

import physbeans.func.*;
import physbeans.math.*;
import physbeans.model.*;
import java.io.*;

/**
 * A physical bean describing a mathematical pendulum with external torque
 * according to the following ODE:<br>
 *    m l^2 &phi;'' + b l^2 &phi;' + m g l sin(&phi;) = M(t)
 * <p>
 * It has the following parameters
 *  <ul>
 * <li>pendulum mass m</li>
 * <li>pendulum length l</li>
 * <li>friction coefficient b</li>
 * <li>gravity constant g</li>
 *  </ul>
 * <p>An arbitrary external torque M can be added via a OneParameterFunction,
 * which is associated with the MathPendulum through the property
 * ExternalTorqueFunction.
 * <br>
 * The torque function may be called several times for one time step due to
 * step adaptation. These calls are done in the stateless mode, so that they
 * don't produce lots of events for external listeners.</p>
 * <p>MathPendulum stores initial values and can be restarted. If an external
 * torque function is used, it is automatically restarted then (sending its new
 * values to potential listeners!).</p>
 */
public class MathPendulum extends SolveODEFunction implements Serializable {
  
  protected double m;
  protected double l;
  protected double b;
  protected double g;
  protected Real1DFunction externalTorqueFunction;
  
  /** Creates a new instance of MathPendulum */
  public MathPendulum() {
    super();
    
    m = 1.0;
    l = 1.0;
    b = 0.0;
    g = Constants.G;
    
    setOde(new PendulumODE());
    externalTorqueFunction = null; // by default, torque = 0.0
    
  }
  
  /**
   * Gets mass m of the pendulum.
   */
  public double getM() {
    return this.m;
  }
  
  /**
   * Sets mass m of the pendulum.
   */
  public void setM(double m) {
    this.m = m;
    restart();
  }
  
  /**
   * Gets length l of pendulum.
   */
  public double getL() {
    return this.l;
  }
  
  /**
   * Sets length l of pendulum.
   */
  public void setL(double l) {
    this.l = l;
    restart();
  }
  
  /**
   * Gets viscous damping coefficient b.
   */
  public double getB() {
    return this.b;
  }
  
  /**
   * Sets viscous damping coefficient b.
   */
  public void setB(double b) {
    this.b = b;
    restart();
  }
  
  /**
   * Gets free fall acceleration g.
   */
  public double getG() {
    return this.g;
  }
  
  /**
   * Sets free fall acceleration g.
   */
  public void setG(double g) {
    this.g = g;
    restart();
  }
  
  /**
   * Gets initial angle phi0.
   */
  public double getPhi0() {
    return getInitialValue(0);
  }
  
  /**
   * Sets initial angle phi0.
   */
  public void setPhi0(double phi0) {
    setInitialValue(0, phi0);
  }
  
  /**
   * Gets initial angular velocity omega0.
   */
  public double getOmega0() {
    return getInitialValue(1);
  }
  
  /**
   * Sets initial angular velocity omega0.
   */
  public void setOmega0(double omega0) {
    setInitialValue(1, omega0);
  }
  
  /**
   * Gets the function that computes time depending values of the external torque.
   */
  public Real1DFunction getExternalTorqueFunction() {
    return this.externalTorqueFunction;
  }
  
  /**
   * Sets the function that computes time depending values of the external torque.
   */
  public void setExternalTorqueFunction(Real1DFunction externalTorqueFunction) {
    this.externalTorqueFunction = externalTorqueFunction;
  }
  
  /**
   * Gets the current value of the angle.<p>
   * Conveniance function to simplify routing
   */
  public double getAngle() {
    return getOutputValues(0);
  }
  
  /**
   * Gets the current value of the angular velocity.<p>
   * Conveniance function to simplify routing
   */
  public double getAngularVelocity() {
    return getOutputValues(1);
  }
  
  class PendulumODE extends ODE {
    public PendulumODE() {
      super(new DVector(1.0, 0.0), 0.0);
    }
    
    public DVector f(DVector x, double t) {
      DVector result = new DVector(2);
      
      // get external torque
      double torque;
      if (externalTorqueFunction != null) {
        torque = externalTorqueFunction.evaluate(t);
      } else {
        torque = 0.0;
      }
      
      result.set(0, x.get(1));
      result.set(1, torque/(m*l*l) - (g/l)*Math.sin(x.get(0)) - (b/m)*x.get(1));
      return result;
    }
    
  }
}
