package physbeans.phys;

import physbeans.func.*;
import physbeans.math.*;
import static java.lang.Math.*;

/**
 * Function that computes mean values of the observables x, y, px, py for
 * a state psi defined by a ComplexGrid2DScalarField.
 * Optionally it computes the values of the uncertainties Delta x etc. and the
 * energy.
 */
public class MeanValueObserver extends GenericNdFunction {

  protected ComplexGrid2DScalarField psi;
  protected boolean computeUncertainties, computeEnergy;
  protected DVector uncertainties;
  protected double probability;
  protected double E;

  public MeanValueObserver() {
    super(0, 4);

    computeUncertainties = false;
    computeEnergy = false;
    uncertainties = new DVector(4);
    probability = 1.0;
    E = 0.0;
  }

  protected DVector computeFunction(DVector in) {
    if (psi == null) {
      return new DVector(4);
    }

    double x = 0.0;              // <x>
    double y = 0.0;              // <y>
    double px = 0.0;             // <px>
    double py = 0.0;             // <py>
    double x2 = 0.0;             // <x*x>
    double y2 = 0.0;             // <y*y>
    double px2 = 0.0;            // <px*px>
    double py2 = 0.0;            // <py*py>
    double norm2 = 0.0;          // <psi, psi>
    double Vxy = 0.0;            // <V>

    int nx = psi.getGridSizeX();
    int ny = psi.getGridSizeY();
    int bwX = psi.getBoundaryWidthX();
    int bwY = psi.getBoundaryWidthY();
    double dx = psi.getDX();
    double dy = psi.getDY();

    double[][] psiR = psi.getGridReal();
    double[][] psiI = psi.getGridImag();
    double[][] VGrid = null;

    if (computeEnergy) {
      VGrid = ((Schroedinger2D) psi).getPotential().getGrid();
    }

    for (int ix = bwX; ix < nx - bwX; ix++) {
      for (int iy = bwY; iy < ny - bwY; iy++) {
        DVector world = psi.getWorldFromIndex(ix, iy);
        double psixy2 = psiR[iy][ix] * psiR[iy][ix] + psiI[iy][ix] *
                psiI[iy][ix];
        norm2 += psixy2;
        x += world.x() * psixy2;
        y += world.y() * psixy2;

        double dxPsiR = -1 / (12 * dx) * (psiR[iy][ix + 2] - 8 *
                psiR[iy][ix + 1] + 8 * psiR[iy][ix - 1] - psiR[iy][ix - 2]);
        double dxPsiI = -1 / (12 * dx) * (psiI[iy][ix + 2] - 8 *
                psiI[iy][ix + 1] + 8 * psiI[iy][ix - 1] - psiI[iy][ix - 2]);
        // global sign change in dy due to the direction change grid / y
        double dyPsiR = 1 / (12 * dy) * (psiR[iy + 2][ix] - 8 *
                psiR[iy + 1][ix] + 8 * psiR[iy - 1][ix] - psiR[iy - 2][ix]);
        double dyPsiI = 1 / (12 * dy) * (psiI[iy + 2][ix] - 8 *
                psiI[iy + 1][ix] + 8 * psiI[iy - 1][ix] - psiI[iy - 2][ix]);
        px += psiR[iy][ix] * dxPsiI - psiI[iy][ix] * dxPsiR;
        py += psiR[iy][ix] * dyPsiI - psiI[iy][ix] * dyPsiR;

        if (computeUncertainties) {
          x2 += world.x() * world.x() * psixy2;
          y2 += world.y() * world.y() * psixy2;

          double d2xPsiR = -1 / (12 * dx * dx) * (psiR[iy][ix + 2] - 16 *
                  psiR[iy][ix + 1] + 30 * psiR[iy][ix] - 16 * psiR[iy][ix - 1] +
                  psiR[iy][ix - 2]);
          double d2xPsiI = -1 / (12 * dx * dx) * (psiI[iy][ix + 2] - 16 *
                  psiI[iy][ix + 1] + 30 * psiI[iy][ix] - 16 * psiI[iy][ix - 1] +
                  psiI[iy][ix - 2]);
          double d2yPsiR = -1 / (12 * dy * dy) * (psiR[iy + 2][ix] - 16 *
                  psiR[iy + 1][ix] + 30 * psiR[iy][ix] - 16 * psiR[iy - 1][ix] +
                  psiR[iy - 2][ix]);
          double d2yPsiI = -1 / (12 * dy * dy) * (psiI[iy + 2][ix] - 16 *
                  psiI[iy + 1][ix] + 30 * psiI[iy][ix] - 16 * psiI[iy - 1][ix] +
                  psiI[iy - 2][ix]);
          px2 += psiR[iy][ix] * d2xPsiR + psiI[iy][ix] * d2xPsiI;
          py2 += psiR[iy][ix] * d2yPsiR + psiI[iy][ix] * d2yPsiI;

          if (computeEnergy) {
            Vxy += VGrid[iy][ix] * psixy2;
          }
        }
      }
    }

    x /= norm2;
    y /= norm2;
    px *= Constants.HBAR / norm2;
    py *= Constants.HBAR / norm2;
    probability = norm2 * dx * dy;

    if (computeUncertainties) {
      x2 /= norm2;
      y2 /= norm2;
      px2 *= -Constants.HBAR * Constants.HBAR / norm2;
      py2 *= -Constants.HBAR * Constants.HBAR / norm2;

      // delta x etc ausrechnen
      uncertainties.set(0, sqrt(x2 - x * x));
      uncertainties.set(1, sqrt(y2 - y * y));
      uncertainties.set(2, sqrt(px2 - px * px));
      uncertainties.set(3, sqrt(py2 - py * py));

      if (computeEnergy) {
        E = (px2 + py2) / (2 * Constants.ME) + Vxy / norm2;
      }
    }


    return new DVector(x, y, px, py);
  }

  /**
   * Gets value of computeUncertainties flag.
   * If true, values for Delta x etc. are computed.
   */
  public boolean isComputeUncertainties() {
    return computeUncertainties;
  }

  /**
   * Sets value of computeUncertainties flag.
   * If true, values for Delta x etc. are computed.
   */
  public void setComputeUncertainties(boolean b) {
    computeUncertainties = b;
    inform();
  }

  /**
   * Gets value of computeEnergy flag.
   * If true, the mean of the energy is computed as well as the uncertainties.
   */
  public boolean isComputeEnergy() {
    return computeEnergy;
  }

  /**
   * Sets value of computeEnergy flag.
   * If true, the mean of the energy is computed as well as the uncertainties.
   */
  public void setComputeEnergy(boolean b) {
    computeEnergy = b;
    if (computeEnergy) {
      computeUncertainties = true;
    }
    inform();
  }

  /**
   * Gets the value of the uncertainties for x, y, px, py.
   */
  public DVector getUncertainties() {
    return uncertainties;
  }

  /**
   * Gets the mean value of the energy.
   */
  public double getEnergy() {
    return E;
  }

  /**
   * Gets the norm squared of the wave function.
   * It should be 1 at the beginning, but decreases, if the particle
   * leaves the computational area.
   */
  public double getProbability() {
    return probability;
  }

  /**
   * Sets the wave function that will be measured.
   * This is defined as the input function, therefore it triggers if
   * autoTriggered is on.
   */
  public void setWaveFunction(ComplexGrid2DScalarField f) {
    psi = f;
    if (autoTriggered) {
      trigger();
    }
  }
}
