package physbeans.phys;

/**
 * A PlaneLightSource creates a bundle of parallel rays, originating from a
 * straight line segment at a given position. The bundle has a defined direction
 * and consists of a number of equally spaced rays.
 */
public class PlaneLightSource extends LightSource {
  
  protected double width;
  protected double angle;
  protected int nRays;
  
  /** Creates a default PlaneLightSource */
  public PlaneLightSource() {
    super();
    width = 1.0;
    angle = 0.0;
    nRays = 5;
    trigger();
  }
  
  /**
   * Gets the width of the parallel light ray bundle.
   */
  public double getWidth() {
    return this.width;
  }
  
  /**
   * Sets the width of the parallel light ray bundle.
   */
  public void setWidth(double width) {
    this.width = width;
    trigger();
  }
  
  /**
   * Gets the angle of the light ray bundle against the optical axis.
   */
  public double getAngle() {
    return this.angle;
  }
  
  /**
   * Sets the angle of the light ray bundle against the optical axis.
   */
  public void setAngle(double angle) {
    this.angle = angle;
    trigger();
  }
  
  /**
   * Gets the number of rays in the bundle.
   */
  public int getNRays() {
    return this.nRays;
  }
  
  /**
   * Sets the number of rays in the bundle.
   * If the number is <= 0, a minimal bundle with one ray is produced.
   */
  public void setNRays(int n) {
    nRays = Math.max(n, 1);
    trigger();
  }
  
  protected void computeRays() {
    rays = new LightRayBundle();
    if (nRays > 1) {
      double sina = Math.sin(angle);
      double cosa = Math.cos(angle);
      for (int i = 0; i < nRays; i++) {
        double x = pos + width*sina*(0.5 - i/(nRays - 1.0));
        double y = height - width*cosa*(0.5 - i/(nRays - 1.0));
        rays.addRay(new LightRay(x, y, angle));
      }
    } else {    // minimum: one ray
      rays.addRay(new LightRay(pos, height, angle));
    }
  }
}
