package physbeans.phys;

import physbeans.math.*;
import java.awt.*;

/**
 * Sets initial values for eigen modes of a rectangular membrane.<p>
 * An eigen mode is defined by two integers (nx, ny) giving the number of
 * maxima in x and y direction.
 * The size of the rectangle can be defined arbitrarily. If the RippleTank
 * provides corresponding walls, the eigen modes will result. If not, the
 * given initial values will disperse.
 */
public class RectangularModeWaveSource extends InitializingWaveSource {
  
  protected int nx, ny;
  protected DVector vllc, vruc;
  protected double a;
  
  /** Creates a new instance of RectangularModeWaveSource */
  public RectangularModeWaveSource() {
    super();
    nx = 2;
    ny = 3;
    vllc = new DVector(-1.0, -1.0);
    vruc = new DVector(1.0, 1.0);
    a = 1.0;
  }
  
  protected void setGrid(RippleTank tank) {
    double[][] grid = tank.getGrid();
    
    Point pllc = tank.getIndex(vllc);
    Point pruc = tank.getIndex(vruc);
    int sx = pruc.x - pllc.x;
    int sy = pllc.y - pruc.y;
    double kx = Math.PI*nx/sx;
    double ky = Math.PI*ny/sy;
    
    for (int i = 0; i <= sx; i++) {
      for (int j = 0; j <= sy; j++) {
        grid[j + pruc.y][i + pllc.x] = a*Math.sin(kx*i)*Math.sin(ky*j);
      }
    }
  }
  
  /**
   * Gets the number of the mode in x direction.
   */
  public int getModeNumberX() {
    return nx;
  }
  
  /**
   * Sets the number of the mode in x direction.
   */
  public void setModeNumberX(int n) {
    nx = n;
  }
  
  /**
   * Gets the number of the mode in y direction.
   */
  public int getModeNumberY() {
    return ny;
  }
  
  /**
   * Sets the number of the mode in y direction.
   */
  public void setModeNumberY(int n) {
    ny = n;
  }
  
  /**
   * Gets the lower left corner of the wall.
   */
  public DVector getLowerLeftCorner() {
    return vllc;
  }
  
  /**
   * Sets the lower left corner of the wall.<br>
   * If the new position is to the right or upwards of the right upper corner,
   * the corresponding coordinates are swapped automatically.
   */
  public void setLowerLeftCorner(DVector v) {
    vllc.set(0, Math.min(v.x(), vruc.x()));
    vruc.set(0, Math.max(v.x(), vruc.x()));
    vllc.set(1, Math.min(v.y(), vruc.y()));
    vruc.set(1, Math.max(v.y(), vruc.y()));
  }
  
  /**
   * Gets the upper right corner of the wall.
   */
  public DVector getUpperRightCorner() {
    return vruc;
  }
  
  /**
   * Sets the upper right corner of the wall.<br>
   * If the new position is to the left or downwards of the lower left corner,
   * the corresponding coordinates are swapped automatically.
   */
  public void setUpperRightCorner(DVector v) {
    vllc.set(0, Math.min(v.x(), vllc.x()));
    vruc.set(0, Math.max(v.x(), vllc.x()));
    vllc.set(1, Math.min(v.y(), vllc.y()));
    vruc.set(1, Math.max(v.y(), vllc.y()));
  }
  
  /**
   * Gets the amplitude of the eigen mode.
   */
  public double getAmplitude() {
    return a;
  }
  
  /**
   * Sets the amplitude of the eigen mode.
   */
  public void setAmplitude(double aNew) {
    a = aNew;
  }
}
