package physbeans.phys;

import physbeans.model.*;
import physbeans.math.*;

/**
 * Potential function for a two-dimensional Schroedinger system
 */
public abstract class Schroedinger2DPotential extends PhysModel implements GridWithBoundary {

  protected Schroedinger2D field;
  protected double[][] grid;

  /** Creates a new instance of Schroedinger2DPotential */
  public Schroedinger2DPotential() {
    super();
  }

  /**
   * Sets up the potential grid.<br>
   * This has to be called each time the field dimensions change.
   */
  public void setup(Schroedinger2D f) {
    if (f != null) {
      field = f;
      createGrid();
      computeGrid();
      trigger();
    }
  }

  /**
   * Creates grid using the dimensions of the field.
   */
  protected void createGrid() {
    if (field != null) {
      int nx = field.getGridSizeX();
      int ny = field.getGridSizeY();

      if ((grid == null) || (grid[0].length != nx) || (grid.length != ny)) {
        grid = new double[ny][nx];
      }
    }
  }

  /**
   * Computes the potential on the grid points.
   */
  protected void computeGrid() {
    if (field != null && grid != null) {
      int nx = field.getGridSizeX();
      int ny = field.getGridSizeY();
      int bwX = field.getBoundaryWidthX();
      int bwY = field.getBoundaryWidthY();
      double xMin = field.getMinWorld().x();
      double yMin = field.getMinWorld().y();
      double dx = field.getDX();
      double dy = field.getDY();

      for (int ix = bwX; ix < nx - bwX; ix++) {
        for (int iy = bwY; iy < ny - bwY; iy++) {
          double Vxy = getScalarValue(xMin + (ix - bwX) * dx, yMin + (iy - bwY) *
                  dy);
          grid[iy][ix] = Vxy;
        }
      }
    }
  }

  /**
   * Function that defines the potential.
   */
  abstract protected double getScalarValue(double x, double y);

  /**
   * Gets the grid of potential values.
   */
  public double[][] getGrid() {
    return grid;
  }

  // functions to implement the interface
  // grid dimensions are given by the underlying field
  public int getBoundaryWidthX() {
    if (field != null) {
      return field.getBoundaryWidthX();
    } else {
      return 0;
    }
  }

  public int getBoundaryWidthY() {
    if (field != null) {
      return field.getBoundaryWidthY();
    } else {
      return 0;
    }
  }

  public int getGridSizeX() {
    if (field != null) {
      return field.getGridSizeX();
    } else {
      return 0;
    }
  }

  public int getGridSizeY() {
    if (field != null) {
      return field.getGridSizeY();
    } else {
      return 0;
    }
  }

  /**
   * Gets vector (xMin, yMin).
   */
  public DVector getMinWorld() {
    if (field != null) {
      return field.getMinWorld();
    } else {
      return null;
    }
  }

  /**
   * Gets vector (xMax, yMax).
   */
  public DVector getMaxWorld() {
    if (field != null) {
      return field.getMaxWorld();
    } else {
      return null;
    }
  }
}
