package physbeans.phys;

/**
 * A thermodynamic system whose state can be changed.
 * <P>Thermodynamic systems are often controlled externally, f.i. in a cyclic
 * process. Triggering is then usually not automatic, but controlled by the
 * external process.
 */
public abstract class ThermodynamicSystem extends PhysModel {
  
  protected boolean autoTriggered;
  
  public ThermodynamicSystem() {
    super();
    autoTriggered = true;
  }
  
  /**
   * change the state according to a polytrope with exponent n. The endpoint of
   * the state change is given by the value val of a ThermodynamicStateVariable
   * (p, T, V).
   */
  public abstract void changeState(double n, ThermodynamicStateVariable tsv, double val);
  
  /**
   * Gets the current value of a thermodynamic state variable of type tsv.
   */
  public abstract double getStateValue(ThermodynamicStateVariable tsv);
  
  /*
   * Sets an arbitrary (initial) state in (p, V)
   */
  public abstract void setState(double p, double V);
  
  /*
   * Resets the thermodynamic systems internal state.
   */
  public abstract void restart();
  
  /**
   * Gets value of autoTriggered flag.
   */
  public boolean isAutoTriggered() {
    return autoTriggered;
  }
  
  /**
   * Sets value of autoTrigger flag.<br>
   * true -> each change of state variables leads to a trigger call.
   */
  public void setAutoTriggered(boolean flag) {
    autoTriggered = flag;
  }
  
  /**
   * Trigger only, if autoTriggered is true.
   */ 
  protected void tryTrigger() {
    if (autoTriggered) {
      trigger();
    }
  }
}
