package physbeans.phys;

import physbeans.event.*;

/**
 * Twodimensional potential function implementing a wall.
 * It uses a WaveWall to define the wall position.
 * the potential is (nearly) infinite at the wall, zero elsewhere.
 */
public class Wall2DPotential extends Schroedinger2DPotential {

  protected WaveWall wall;
  protected double val;

  public Wall2DPotential() {
    super();
    val = 1e10;
  }

  /**
   * Gets the currently used wall.
   */
  public WaveWall getWall() {
    return wall;
  }

  /**
   * Defines a new wall.
   */
  public void setWall(WaveWall w) {
    if (w != null) {
      wall = w;
      computeGrid();
      if (field != null) {
        field.computeCachedMatrices();
      }

      // listen to the wall to recompute V when a wall parameter changes
      wall.addTriggerListener(new TriggerListener() {

        public void sourceChanged(TriggerEvent evt) {
          computeGrid();
          if (field != null) {
            field.computeCachedMatrices();
          }
        }
      });
    }
  }

  /**
   * Gets the potential value at the wall.
   */
  public double getWallValue() {
    return val;
  }

  /**
   * Sets the potential value at the wall.
   */
  public void setWallValue(double v) {
    val = v;
    computeGrid();
    if (field != null) {
      field.computeCachedMatrices();
    }
  }

  @Override
  protected void computeGrid() {
    if (field != null && grid != null && wall != null) {
      double[][] wallGrid = wall.getGrid();

      if (wallGrid == null) {
        wall.setup(field);
        wallGrid = wall.getGrid();
      }

      int nx = field.getGridSizeX();
      int ny = field.getGridSizeY();
      int bwX = field.getBoundaryWidthX();
      int bwY = field.getBoundaryWidthY();

      for (int ix = bwX; ix < nx - bwX; ix++) {
        for (int iy = bwY; iy < ny - bwY; iy++) {
          if (wallGrid[iy][ix] == 1.0) {
            grid[iy][ix] = val;
          } else {
            grid[iy][ix] = 0.0;
          }
        }
      }
    }
  }

  protected double getScalarValue(double x, double y) {
    return 0.0;
  }
}
