package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;

import physbeans.inout.ScreenWorldTrafo;
import physbeans.phys.*;
import physbeans.math.*;
import physbeans.model.*;

/**
 * View displaying a PointLightSource and its light rays.
 * Displays the source as a small disk.
 */
public class PointLightSourceView extends View {
  
  protected PointLightSource model;
  protected LightRayBundleView raysView;
  protected Color sourceColor;
  protected double diameter;
  
  
  /** Creates a new instance of PointLightSourceView */
  public PointLightSourceView() {
    raysView = new LightRayBundleView();
    sourceColor = new Color(255, 204, 0);
    diameter = 0.5;
  }
  
  public void paintLayer(Graphics2D g2) {
    if (model == null) {
      return;  // do nothing
    }
    
    double x = model.getPosition();
    double y = model.getHeight();
    double r = diameter/2;
    double ry = r * trafo.getRelativeScaling();
    Ellipse2D circle = new Ellipse2D.Double(x - r, y - ry, 2*r, 2*ry);
    
    raysView.setModel(model.getRays());
    raysView.paintLayer(g2);

    g2.setColor(sourceColor);
    g2.fill(circle);

    g2.setColor(Color.black);
    g2.draw(circle);

  }
  
  /**
   * Gets color of the lines.
   */
  public Color getRayColor() {
    return raysView.getLineColor();
  }
  
  /**
   * Sets color of the lines.
   */
  public void setRayColor(Color lineColor) {
    raysView.setLineColor(lineColor);
    update();
  }
  
  /**
   * Sets trafo between screen and world coordinates.
   */
  public void setTrafo(ScreenWorldTrafo trafo) {
    super.setTrafo(trafo);
    raysView.setTrafo(trafo);
  }
  
  /**
   * Gets the color of the figure representing the source.
   */
  public Color getSourceColor() {
    return this.sourceColor;
  }
  
  /**
   * Sets the color of the figure representing the source.
   */
  public void setSourceColor(Color sourceColor) {
    this.sourceColor = sourceColor;
  }
  
  /**
   * Gets the diameter of the figure representing the source.
   */
  public double getDiameter() {
    return this.diameter;
  }
  
  /**
   * Sets the diameter of the figure representing the source.
   */
  public void setDiameter(double diameter) {
    this.diameter = diameter;
  }
  
  /**
   * Gets underlying model defining the light source to paint.
   */
  public PointLightSource getModel() {
    return model;
  }
  
  /**
   * Sets underlying model defining the light source to paint.
   */
  public void setModel(PointLightSource model) {
    this.model = model;
    update();
  }
}
