package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;
import physbeans.model.*;
import physbeans.math.*;

/**
 * View that draws a vector of 2d points,
 * either given as a Point2dVector or as two DVectors (x and y coordinates),
 * by representing each point by a symbol (dot, crossed circle etc.).
 */
public class PointView extends View implements Serializable {
  
  protected Color pointColor;
  protected Color activePointColor;
  protected Point2dVector points;
  protected DVector x, y;
  protected double r;
  protected boolean paintCross, useTwoVectors;
  
  /**
   * creates PointView with an empty set of points and default values
   */
  public PointView() {
    super();
    useTwoVectors = false;
    paintCross = true;
    points = new Point2dVector();
    x = new DVector(0);
    y = new DVector(0);
    
    pointColor = Color.red;
    activePointColor = Color.green;
    r = 0.5;
  }
  
  /**
   * Draws point at (x,y) as a crossed circle.
   */
  protected void drawPoint(Graphics2D g2, DVector v) {
    double x = v.get(0);
    double y = v.get(1);
    double dx = 0.70710678119*r;   // = cos(45°)
    
    double ry = r * trafo.getRelativeScaling();
    double dy = dx * trafo.getRelativeScaling();
    
    
    Ellipse2D circle = new Ellipse2D.Double(x - r, y - ry, 2*r, 2*ry);
    g2.draw(circle);
    
    if (paintCross) {
      Line2D l1 = new Line2D.Double(x - dx, y - dy, x + dx, y + dy);
      Line2D l2 = new Line2D.Double(x + dx, y - dy, x - dx, y + dy);
      g2.draw(l1);
      g2.draw(l2);
    }
  }
  
  /**
   * Draws all points.
   */
  public void paintLayer(Graphics2D g2) {
    if (useTwoVectors) {
      paintLayerFromVectors(g2);
    } else {
      paintLayerFromPoints(g2);
    }
  }
  
  /**
   * Gets color of points.
   */
  public Color getPointColor() {
    return this.pointColor;
  }
  
  /**
   * sets color of points
   */
  public void setPointColor(Color pointColor) {
    this.pointColor = pointColor;
    update();
  }
  
  /**
   * Gets radius of a point.
   */
  public double getPointRadius() {
    return r;
  }
  
  /**
   * Sets radius of a point.
   */
  public void setPointRadius(double r) {
    this.r = r;
    update();
  }
  
  /**
   * gets underlying model defining the points to paint
   */
  public Point2dVector getModel() {
    return points;
  }
  
  /**
   * sets underlying model defining the points to paint
   */
  public void setModel(Point2dVector model) {
    points = model;
    update();
  }
  
  /**
   * sets underlying model defining the points to paint
   */
  public void setModel(DVector x1, DVector y1) {
    x = x1;
    y = y1;
    update();
  }
  
  /**
   * gets color of "active" point
   */
  public Color getActivePointColor() {
    return this.activePointColor;
  }
  
  /**
   * sets color of "active" point
   */
  public void setActivePointColor(Color activePointColor) {
    this.activePointColor = activePointColor;
    update();
  }
  
  /**
   * Gets switch to choose between Point2DVector or two Vectors for input.
   */
  public boolean isUseTwoVectors() {
    return this.useTwoVectors;
  }
  
  /**
   * Sets switch to choose between Point2DVector or two Vectors for input.
   */
  public void setUseTwoVectors(boolean useTwoVectors) {
    this.useTwoVectors = useTwoVectors;
    update();
  }
  
  /**
   * Gets switch that chooses between simple or crossed circle drawing.
   */
  public boolean isPaintCross() {
    return this.paintCross;
  }
  
  /**
   * Sets switch that chooses between simple or crossed circle drawing.
   */
  public void setPaintCross(boolean paintCross) {
    this.paintCross = paintCross;
    update();
  }
  
  /**
   * paint points using Point2dVector
   */
  protected void paintLayerFromPoints(Graphics2D g2) {
    if (points == null) {
      return;  // do nothing
    }
    
    g2.setColor(pointColor);
    for (int i = 0; i < points.size(); i++) {
      drawPoint(g2, points.getPoint(i));
    }
    
    // simply redraw active point if necessary
    int nActive = points.getActivePoint();
    if ( nActive >= 0) {
      g2.setColor(activePointColor);
      drawPoint(g2, points.getPoint(nActive));
    }
  }
  
  /**
   * paint points using two DVectors
   */
  protected void paintLayerFromVectors(Graphics2D g2) {
    if (x == null || y == null) {
      return;  // do nothing
    }
    
    g2.setColor(pointColor);
    for (int i = 0; i < x.getDimension(); i++) {
      drawPoint(g2, new DVector(x.get(i), y.get(i)));
    }
    
    // no active point when using vectors
  }
  
}
