package physbeans.views;
import physbeans.math.*;

/**
 * Base class of views that hold initial values (given by a DVector)
 * and have a restart method to go back to the initial values.
 */
public abstract class RestartableView extends View {
  
  protected DVector model;
  protected DVector initialValues;
  
  /** Creates a new instance of RestartableView */
  public RestartableView() {
    super();
    model = new DVector(2);
    initialValues = new DVector(2);
  }
  
  /**
   * Gets underlying model defining the input parameters.
   */
  public DVector getModel() {
    return model;
  }
  
  /**
   * Sets underlying model defining the input parameters.
   */
  public void setModel(DVector m) {
    model = m;
    update();
  }
  
  /**
   * Gets initial values of one model parameter.
   */
  public double getInitialValue(int index) {
    return initialValues.get(index);
  }
  
  /**
   * Gets initial values of all model parameters.
   */
  public DVector getInitialValues() {
    return initialValues;
  }
  
  /**
   * Sets one of the initial values and the
   *   corresponding actual value.
   */
  public void setInitialValue(int index, double val) {
    initialValues.set(index, val);
    model.set(index, val);
    update();
  }
  
  /**
   * Sets all initial values.<br>
   *  Sets the actual values to the initial values.
   */
  public void setInitialValues(DVector vals) {
    initialValues = vals;
    model = vals.copy();
    update();
  }
  
  /**
   * Resets the model to the initial values and redraws.
   */
  public void restart() {
    model = initialValues.copy();
    update();
  }
}
