package physbeans.views;

import java.awt.*;
import java.awt.geom.*;
import physbeans.math.*;
import physbeans.model.*;

/**
 * View that draws a regular grid of vector arrows representing a vector field.<br>
 * If the length of an arrow is too large a marker may be drawn instead.
 */
public class VectorFieldView extends View {
  
  protected VectorField vectorField;
  protected int nGridX;
  protected int nGridY;
  protected double padding;
  protected double vectorScale;
  protected double maxArLength;
  protected boolean showMarker;
  protected Color arrowColor;
  protected Color markerColor;
  
  /**
   * creates default VectorFieldView
   */
  public VectorFieldView() {
    super();
    nGridX = 5;
    nGridY = 5;
    padding = 0.05;
    
    arrowColor = Color.black;
    vectorScale = 1.0;
    
    markerColor = Color.red;
    maxArLength = 2.0;
    showMarker = false;
  }
  
  /**
   * draws an arrow from p to p + dp
   */
  protected void drawArrow(Graphics2D g, Point2D p, Point2D dp) {
    // values relative to arrow length
    double arrowTipWidth = 0.2;
    double arrowTipLength = 0.2;
    
    //generate a standard arrow of length
    GeneralPath arrow1 = new GeneralPath();
    arrow1.moveTo(0.0f, 0.0f);
    arrow1.lineTo(1.0f, 0.0f);
    arrow1.lineTo(1.0f - (float)arrowTipLength, (float)arrowTipWidth);
    arrow1.moveTo(1.0f, 0.0f);
    arrow1.lineTo(1.0f - (float)arrowTipLength, -(float)arrowTipWidth);
    
    // scale arrow to its length
    double length = dp.distance(0.0, 0.0);
    AffineTransform dimensioning = AffineTransform.getScaleInstance(length, length);
    arrow1.transform(dimensioning);
    
    //rotate arrow into the right direction
    double alpha = Math.atan2(dp.getY(), dp.getX());
    dimensioning = AffineTransform.getRotateInstance(alpha, 0,0);
    arrow1.transform(dimensioning);
    
    //translate arrow to origin p
    dimensioning = AffineTransform.getTranslateInstance(p.getX(), p.getY());
    arrow1.transform(dimensioning);
    
    g.setColor(arrowColor);
    g.draw(arrow1);
  }
  
  /**
   * Draws a marker at given position.
   */
  protected void drawMarker(Graphics2D g, Point2D p) {
    double x = p.getX();
    double y = p.getY();
    
    double r = 5*trafo.getScalingX();              // 5 pixel radius
    double dx = 0.70710678119*r;                   // = cos(45°)
    double ry = r * trafo.getRelativeScaling();    // make it a circle
    double dy = dx * trafo.getRelativeScaling();
    
    Ellipse2D circle = new Ellipse2D.Double(x - r, y - ry, 2*r, 2*ry);
    Line2D l1 = new Line2D.Double(x - dx, y - dy, x + dx, y + dy);
    Line2D l2 = new Line2D.Double(x + dx, y - dy, x - dx, y + dy);
    
    g.setColor(markerColor);
    g.draw(circle);
    g.draw(l1);
    g.draw(l2);
  }
  
  /**
   * paint a grid of arrows
   */
  public void paintLayer(Graphics2D g) {
    if (vectorField == null) {
      return;
    }
    
    // compute grid spacings
    double maxX = trafo.getMaxX();
    double minX = trafo.getMinX();
    double maxY = trafo.getMaxY();
    double minY = trafo.getMinY();
    double ds = (maxX - minX - 2*padding)/(nGridX - 1);
    double dt = (maxY - minY - 2*padding)/(nGridY-1);
    
    // loop over all grid points
    Point2D p;                   // start point of an arrow
    DVector v;
    double length;
    double minLx = trafo.getScalingX();
    double minLy = trafo.getScalingY();
    for (int i = 0; i < nGridX; i++) {
      for (int j = 0; j < nGridY; j++) {
        p = new Point2D.Double(minX + i*ds + padding, minY + j*dt + padding);
        v = vectorField.getVectorValue(new DVector(p.getX(),p.getY()));
        v.timesEquals(vectorScale);
        length = v.norm();
        if (length > maxArLength) {
          if (showMarker) {
            drawMarker(g, p);
          }
        } else if (length < minLx) {
          Rectangle2D pix = new Rectangle2D.Double(p.getX(), p.getY(), minLx, minLy);
          g.fill(pix);
          // draw at least a pixel
        } else {
          drawArrow(g, p, new Point2D.Double(v.x(), v.y()));
        }
      }
    }
  }
  
  /**
   * Gets underlying model defining the vector field to draw.
   */
  public VectorField getModel() {
    return vectorField;
  }
  
  /**
   * Sets underlying model defining the vector field to draw.
   */
  public void setModel(VectorField model) {
    vectorField = model;
    update();
  }
  
  /**
   * Gets number of grid points in x direction.
   */
  public int getNGridX() {
    return nGridX;
  }
  
  /**
   * Sets number of grid points in x direction.
   */
  public void setNGridX(int nGridX) {
    this.nGridX = nGridX;
    update();
  }
  
  /**
   * Gets number of grid points in y direction.
   */
  public int getNGridY() {
    return nGridY;
  }
  
  /**
   * Sets number of grid points in y direction.
   */
  public void setNGridY(int nGridY) {
    this.nGridY = nGridY;
    update();
  }
  
  /**
   * Gets padding space around grid.
   */
  public double getPadding() {
    return padding;
  }
  
  /**
   * Sets padding space around grid.
   */
  public void setPadding(double padding) {
    this.padding = padding;
    update();
  }
  
  /**
   * Gets the pixel size of a vector of length 1.0.<br>
   * Value is double to allow for finetuning of lenghts.
   */
  public double getVectorScale() {
    return vectorScale;
  }
  
  /**
   * Sets the pixel size of a vector of length 1.0.<br>
   * Value is double to allow for finetuning of lenghts.
   */
  public void setVectorScale(double vectorScale) {
    this.vectorScale = vectorScale;
    update();
  }
  
  /**
   * Gets showMarker flag.<br>
   * If true the marker is used instead of very large arrows.
   */
  public boolean isShowMarker() {
    return this.showMarker;
  }
  
  /**
   * Sets showMarker flag.<br>
   * If true the marker is used instead of very large arrows.
   */
  public void setShowMarker(boolean showMarker) {
    this.showMarker = showMarker;
    update();
  }
  
  /**
   * Gets the maximal value of a vector that is drawn with an arrow. <br>
   * If the showMarker property is set larger, values are displayed with a marker,
   * otherwise this value is ignored.
   */
  public double getMaxArrowLength() {
    return maxArLength;
  }
  
  /**
   * Sets the maximal value of a vector that is drawn with an arrow.
   */
  public void setMaxArrowLength(double max) {
    maxArLength = max;
    update();
  }
  
  /**
   * Gets color used for the arrows.
   */
  public Color getArrowColor() {
    return this.arrowColor;
  }
  
  /**
   * Sets color used for the arrows.
   */
  public void setArrowColor(Color arrowColor) {
    this.arrowColor = arrowColor;
    update();
  }
  
  /**
   * Gets color used for the marker.
   */
  public Color getMarkerColor() {
    return this.markerColor;
  }
  
  /**
   * Sets color used for the marker.
   */
  public void setMarkerColor(Color markerColor) {
    this.markerColor = markerColor;
    update();
  }
}

